@@ -1,14 +1,14 @@
 // N-D Array  manipulations.
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -494,7 +493,7 @@ ComplexNDArray::operator ! (void) const
   return b;
 }
 
-// XXX FIXME XXX -- this is not quite the right thing.
+// FIXME -- this is not quite the right thing.
 
 bool
 ComplexNDArray::any_element_is_inf_or_nan (void) const
@@ -594,10 +593,10 @@ ComplexNDArray::too_large_for_float (void) const
       double r_val = std::real (val);
       double i_val = std::imag (val);
 
-      if (r_val > FLT_MAX
-	  || i_val > FLT_MAX
-	  || r_val < FLT_MIN
-	  || i_val < FLT_MIN)
+      if ((! (xisnan (r_val) || xisinf (r_val))
+	   && fabs (r_val) > FLT_MAX)
+	  || (! (xisnan (i_val) || xisinf (i_val))
+	      && fabs (i_val) > FLT_MAX))
 	return true;
     }
 
@@ -735,7 +734,7 @@ ComplexNDArray::max (ArrayN<octave_idx_type>& idx_arg, int dim) const
 	{
 	  tmp_max = elem (idx_j * x_stride + x_offset);
 	  
-	  if (! octave_is_NaN_or_NA (tmp_max))
+	  if (! xisnan (tmp_max))
 	    {
 	      abs_max = std::abs(tmp_max);
 	      break;
@@ -746,7 +745,7 @@ ComplexNDArray::max (ArrayN<octave_idx_type>& idx_arg, int dim) const
 	{
 	  Complex tmp = elem (j * x_stride + x_offset);
 
-	  if (octave_is_NaN_or_NA (tmp))
+	  if (xisnan (tmp))
 	    continue;
 
 	  double abs_tmp = std::abs (tmp);
@@ -759,7 +758,7 @@ ComplexNDArray::max (ArrayN<octave_idx_type>& idx_arg, int dim) const
 	    }
 	}
 
-      if (octave_is_NaN_or_NA (tmp_max))
+      if (xisnan (tmp_max))
 	{
 	  result.elem (i) = Complex_NaN_result;
 	  idx_arg.elem (i) = 0;
@@ -827,7 +826,7 @@ ComplexNDArray::min (ArrayN<octave_idx_type>& idx_arg, int dim) const
 	{
 	  tmp_min = elem (idx_j * x_stride + x_offset);
 	  
-	  if (! octave_is_NaN_or_NA (tmp_min))
+	  if (! xisnan (tmp_min))
 	    {
 	      abs_min = std::abs(tmp_min);
 	      break;
@@ -838,7 +837,7 @@ ComplexNDArray::min (ArrayN<octave_idx_type>& idx_arg, int dim) const
 	{
 	  Complex tmp = elem (j * x_stride + x_offset);
 
-	  if (octave_is_NaN_or_NA (tmp))
+	  if (xisnan (tmp))
 	    continue;
 
 	  double abs_tmp = std::abs (tmp);
@@ -851,7 +850,7 @@ ComplexNDArray::min (ArrayN<octave_idx_type>& idx_arg, int dim) const
 	    }
 	}
 
-      if (octave_is_NaN_or_NA (tmp_min))
+      if (xisnan (tmp_min))
 	{
 	  result.elem (i) = Complex_NaN_result;
 	  idx_arg.elem (i) = 0;
@@ -1026,7 +1025,7 @@ operator >> (std::istream& is, ComplexNDArray& a)
   return is;
 }
 
-// XXX FIXME XXX -- it would be nice to share code among the min/max
+// FIXME -- it would be nice to share code among the min/max
 // functions below.
 
 #define EMPTY_RETURN_CHECK(T) \
