@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+              2002, 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -28,10 +28,11 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <cfloat>
 #include <cmath>
 
+#include <sstream>
+
 #include "DASSL.h"
 #include "f77-fcn.h"
 #include "lo-error.h"
-#include "lo-sstream.h"
 #include "quit.h"
 
 typedef octave_idx_type (*dassl_fcn_ptr) (const double&, const double*, const double*,
@@ -62,7 +63,7 @@ ddassl_f (const double& time, const double *state, const double *deriv,
 {
   BEGIN_INTERRUPT_WITH_EXCEPTIONS;
 
-  // XXX FIXME XXX -- would be nice to avoid copying the data.
+  // FIXME -- would be nice to avoid copying the data.
 
   ColumnVector tmp_deriv (nn);
   ColumnVector tmp_state (nn);
@@ -98,7 +99,7 @@ ddassl_j (const double& time, const double *state, const double *deriv,
 {
   BEGIN_INTERRUPT_WITH_EXCEPTIONS;
 
-  // XXX FIXME XXX -- would be nice to avoid copying the data.
+  // FIXME -- would be nice to avoid copying the data.
 
   ColumnVector tmp_state (nn);
   ColumnVector tmp_deriv (nn);
@@ -503,10 +504,9 @@ DASSL::error_message (void) const
 {
   std::string retval;
 
-  OSSTREAM buf;
-  buf << t << OSSTREAM_ENDS;
-  std::string t_curr = OSSTREAM_STR (buf);
-  OSSTREAM_FREEZE (buf);
+  std::ostringstream buf;
+  buf << t;
+  std::string t_curr = buf.str ();
 
   switch (istate)
     {
