@@ -1,13 +1,15 @@
 // Template sparse array class
 /*
 
-Copyright (C) 2004 David Bateman
-Copyright (C) 1998-2004 Andy Adler
+Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
+
+This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +17,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -730,14 +731,26 @@ Sparse<T>
 Sparse<T>::reshape (const dim_vector& new_dims) const
 {
   Sparse<T> retval;
+  dim_vector dims2 = new_dims;
 
-  if (dimensions != new_dims)
+  if (dims2.length () > 2)
     {
-      if (dimensions.numel () == new_dims.numel ())
+      (*current_liboctave_warning_handler)
+	("reshape: sparse reshape to N-d array smashes dims");
+
+      for (octave_idx_type i = 2; i < dims2.length(); i++)
+	dims2(1) *= dims2(i);
+
+      dims2.resize (2);
+    }
+
+  if (dimensions != dims2)
+    {
+      if (dimensions.numel () == dims2.numel ())
 	{
 	  octave_idx_type new_nnz = nnz ();
-	  octave_idx_type new_nr = new_dims (0);
-	  octave_idx_type new_nc = new_dims (1);
+	  octave_idx_type new_nr = dims2 (0);
+	  octave_idx_type new_nc = dims2 (1);
 	  octave_idx_type old_nr = rows ();
 	  octave_idx_type old_nc = cols ();
 	  retval = Sparse<T> (new_nr, new_nc, new_nnz);
@@ -772,46 +785,28 @@ template <class T>
 Sparse<T>
 Sparse<T>::permute (const Array<octave_idx_type>& perm_vec, bool) const
 {
-  dim_vector dv = dims ();
-  dim_vector dv_new;
+  // The only valid permutations of a sparse array are [1, 2] and [2, 1].
 
-  octave_idx_type nd = dv.length ();
+  bool fail = false;
+  bool trans = false;
 
-  dv_new.resize (nd);
-
-  // Need this array to check for identical elements in permutation array.
-  Array<bool> checked (nd, false);
-
-  // Find dimension vector of permuted array.
-  for (octave_idx_type i = 0; i < nd; i++)
+  if (perm_vec.length () == 2)
     {
-      octave_idx_type perm_el = perm_vec.elem (i);
-
-      if (perm_el > dv.length () || perm_el < 1)
-	{
-	  (*current_liboctave_error_handler)
-	    ("permutation vector contains an invalid element");
-
-	  return Sparse<T> ();
-	}
-
-      if (checked.elem(perm_el - 1))
-	{
-	  (*current_liboctave_error_handler)
-	    ("PERM cannot contain identical elements");
-
-	  return Sparse<T> ();
-	}
+      if (perm_vec(0) == 0 && perm_vec(1) == 1)
+	/* do nothing */;
+      else if (perm_vec(0) == 1 && perm_vec(1) == 0)
+	trans = true;
       else
-	checked.elem(perm_el - 1) = true;
-
-      dv_new (i) = dv (perm_el - 1);
+	fail = true;
     }
-
-  if (dv_new == dv)
-    return *this;
   else
-    return transpose ();
+    fail = true;
+
+  if (fail)
+    (*current_liboctave_error_handler)
+      ("permutation vector contains an invalid element");
+
+  return trans ? this->transpose () : *this;
 }
 
 template <class T>
@@ -1817,6 +1812,13 @@ Sparse<T>::index (idx_vector& idx_arg, int resize_ok) const
   return retval;
 }
 
+struct 
+idx_node 
+{
+  octave_idx_type i;
+  struct idx_node *next;
+};		    
+
 template <class T>
 Sparse<T>
 Sparse<T>::index (idx_vector& idx_i, idx_vector& idx_j, int resize_ok) const
@@ -1909,6 +1911,8 @@ Sparse<T>::index (idx_vector& idx_i, idx_vector& idx_j, int resize_ok) const
 		      octave_idx_type jj = idx_j.elem (j);
 		      for (octave_idx_type i = cidx(jj); i < cidx(jj+1); i++)
 			{
+			  OCTAVE_QUIT;
+
 			  octave_idx_type ii = itmp [ridx(i)];
 			  if (ii >= 0)
 			    {
@@ -1925,56 +1929,109 @@ Sparse<T>::index (idx_vector& idx_i, idx_vector& idx_j, int resize_ok) const
 		}
 	      else
 		{
+		  OCTAVE_LOCAL_BUFFER (struct idx_node, nodes, n); 
+		  OCTAVE_LOCAL_BUFFER (octave_idx_type, start_nodes, nr); 
+
+		  for (octave_idx_type i = 0; i < nr; i++)
+		    start_nodes[i] = -1;
+
+		  for (octave_idx_type i = 0; i < n; i++)
+		    {
+		      octave_idx_type ii = idx_i.elem (i);
+		      nodes[i].i = i;
+		      nodes[i].next = 0;
+
+		      octave_idx_type node = start_nodes[ii];
+		      if (node == -1)
+			start_nodes[ii] = i;
+		      else
+			{
+			  struct idx_node inode = nodes[node];
+			  while (inode.next)
+			    inode = *inode.next;
+			  inode.next = nodes + i;
+			}
+		    }
+
 		  // First count the number of non-zero elements
 		  octave_idx_type new_nzmx = 0;
 		  for (octave_idx_type j = 0; j < m; j++)
 		    {
 		      octave_idx_type jj = idx_j.elem (j);
-		      for (octave_idx_type i = 0; i < n; i++)
-			{
-			  OCTAVE_QUIT;
 
-			  octave_idx_type ii = idx_i.elem (i);
-			  if (ii < nr && jj < nc)
+		      if (jj < nc)
+			{
+			  for (octave_idx_type i = cidx(jj); 
+			       i < cidx(jj+1); i++)
 			    {
-			      for (octave_idx_type k = cidx(jj); k < cidx(jj+1); k++)
+			      OCTAVE_QUIT;
+
+			      octave_idx_type ii = start_nodes [ridx(i)];
+
+			      if (ii >= 0)
 				{
-				  if (ridx(k) == ii)
-				    new_nzmx++;
-				  if (ridx(k) >= ii)
-				    break;
+				  struct idx_node inode = nodes[ii];
+			      
+				  while (true)
+				    {
+				      if (inode.i >= 0 && 
+					  idx_i.elem (inode.i) < nc)
+					new_nzmx ++;
+				      if (inode.next == 0)
+					break;
+				      else
+					inode = *inode.next;
+				    }
 				}
 			    }
 			}
 		    }
 
+		  std::vector<T> X (n);
 		  retval = Sparse<T> (n, m, new_nzmx);
+		  octave_idx_type *ri = retval.xridx ();
 
 		  octave_idx_type kk = 0;
 		  retval.xcidx(0) = 0;
 		  for (octave_idx_type j = 0; j < m; j++)
 		    {
 		      octave_idx_type jj = idx_j.elem (j);
-		      for (octave_idx_type i = 0; i < n; i++)
+		      if (jj < nc)
 			{
-			  OCTAVE_QUIT;
-
-			  octave_idx_type ii = idx_i.elem (i);
-			  if (ii < nr && jj < nc)
+			  for (octave_idx_type i = cidx(jj); 
+			       i < cidx(jj+1); i++)
 			    {
-			      for (octave_idx_type k = cidx(jj); k < cidx(jj+1); k++)
+			      OCTAVE_QUIT;
+
+			      octave_idx_type ii = start_nodes [ridx(i)];
+
+			      if (ii >= 0)
 				{
-				  if (ridx(k) == ii)
+				  struct idx_node inode = nodes[ii];
+			      
+				  while (true)
 				    {
-				      retval.xdata(kk) = data(k);
-				      retval.xridx(kk++) = i;
+				      if (inode.i >= 0 && 
+					  idx_i.elem (inode.i) < nc)
+					{
+					  X [inode.i] = data (i);
+					  retval.xridx (kk++) = inode.i;
+					}
+
+				      if (inode.next == 0)
+					break;
+				      else
+					inode = *inode.next;
 				    }
-				  if (ridx(k) >= ii)
-				    break;
 				}
 			    }
+			  sort.sort (ri + retval.xcidx (j), 
+				     kk - retval.xcidx (j));
+			  for (octave_idx_type p = retval.xcidx (j); 
+			       p < kk; p++)
+			    retval.xdata (p) = X [retval.xridx (p)]; 
+			  retval.xcidx(j+1) = kk;
 			}
-		      retval.xcidx(j+1) = kk;
 		    }
 		}
 	    }
@@ -2402,6 +2459,10 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
   if (n_idx > 0)
     idx_i = tmp[0];
 
+  // Take a constant copy of lhs. This means that ridx and family won't 
+  // call make_unique.
+  const Sparse<LT> c_lhs (lhs);
+
   if (n_idx == 2)
     {
       octave_idx_type n = idx_i.freeze (lhs_nr, "row", true);
@@ -2459,12 +2520,12 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
 			      
 				  if (ii < lhs_nr)
 				    {
-				      for (octave_idx_type k = lhs.cidx(jj); 
-					   k < lhs.cidx(jj+1); k++)
+				      for (octave_idx_type k = c_lhs.cidx(jj); 
+					   k < c_lhs.cidx(jj+1); k++)
 					{
-					  if (lhs.ridx(k) == ii)
+					  if (c_lhs.ridx(k) == ii)
 					    new_nzmx--;
-					  if (lhs.ridx(k) >= ii)
+					  if (c_lhs.ridx(k) >= ii)
 					    break;
 					}
 				    }
@@ -2489,11 +2550,11 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
 			      octave_idx_type ii = idx_i.elem (iii);
 			      octave_idx_type ppp = 0;
 			      octave_idx_type ppi = (j >= lhs_nc ? 0 : 
-						     lhs.cidx(j+1) - 
-						     lhs.cidx(j));
+						     c_lhs.cidx(j+1) - 
+						     c_lhs.cidx(j));
 			      octave_idx_type pp = (ppp < ppi ? 
-						    lhs.ridx(lhs.cidx(j)+ppp) :
-						    new_nr);
+					c_lhs.ridx(c_lhs.cidx(j)+ppp) :
+					new_nr);
 			      while (ppp < ppi || iii < n)
 				{
 				  if (iii < n && ii <= pp)
@@ -2504,28 +2565,28 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
 					  stmp.ridx(kk++) = ii;
 					}
 				      if (ii == pp)
-					pp = (++ppp < ppi ? lhs.ridx(lhs.cidx(j)+ppp) : new_nr);					
+					pp = (++ppp < ppi ? c_lhs.ridx(c_lhs.cidx(j)+ppp) : new_nr);					
 				      if (++iii < n)
 					ii = idx_i.elem(iii);
 				    }
 				  else
 				    {
 				      stmp.data(kk) = 
-					lhs.data(lhs.cidx(j)+ppp);
+					c_lhs.data(c_lhs.cidx(j)+ppp);
 				      stmp.ridx(kk++) = pp;
-				      pp = (++ppp < ppi ? lhs.ridx(lhs.cidx(j)+ppp) : new_nr);
+				      pp = (++ppp < ppi ? c_lhs.ridx(c_lhs.cidx(j)+ppp) : new_nr);
 				    }
 				}
 			      if (++jji < m)
 				jj = idx_j.elem(jji);
 			    }
-			  else if (j < lhs.cols()) 
+			  else if (j < lhs_nc) 
 			    {
-			      for (octave_idx_type i = lhs.cidx(j); 
-				   i < lhs.cidx(j+1); i++)
+			      for (octave_idx_type i = c_lhs.cidx(j); 
+				   i < c_lhs.cidx(j+1); i++)
 				{
-				  stmp.data(kk) = lhs.data(i);
-				  stmp.ridx(kk++) = lhs.ridx(i);
+				  stmp.data(kk) = c_lhs.data(i);
+				  stmp.ridx(kk++) = c_lhs.ridx(i);
 				}
 			    }
 			  stmp.cidx(j+1) = kk;
@@ -2642,11 +2703,11 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
 			      octave_idx_type ii = idx_i.elem (iii);
 			      octave_idx_type ppp = 0;
 			      octave_idx_type ppi = (j >= lhs_nc ? 0 : 
-						     lhs.cidx(j+1) - 
-						     lhs.cidx(j));
+						     c_lhs.cidx(j+1) - 
+						     c_lhs.cidx(j));
 			      octave_idx_type pp = (ppp < ppi ? 
-						    lhs.ridx(lhs.cidx(j)+ppp) :
-						    new_nr);
+						c_lhs.ridx(c_lhs.cidx(j)+ppp) :
+						new_nr);
 			      while (ppp < ppi || iii < n)
 				{
 				  if (iii < n && ii <= pp)
@@ -2659,28 +2720,28 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
 					  stmp.ridx(kk++) = ii;
 					}
 				      if (ii == pp)
-					pp = (++ppp < ppi ? lhs.ridx(lhs.cidx(j)+ppp) : new_nr);					
+					pp = (++ppp < ppi ? c_lhs.ridx(c_lhs.cidx(j)+ppp) : new_nr);					
 				      if (++iii < n)
 					ii = idx_i.elem(iii);
 				    }
 				  else
 				    {
 				      stmp.data(kk) = 
-					lhs.data(lhs.cidx(j)+ppp);
+					c_lhs.data(c_lhs.cidx(j)+ppp);
 				      stmp.ridx(kk++) = pp;
-				      pp = (++ppp < ppi ? lhs.ridx(lhs.cidx(j)+ppp) : new_nr);
+				      pp = (++ppp < ppi ? c_lhs.ridx(c_lhs.cidx(j)+ppp) : new_nr);
 				    }
 				}
 			      if (++jji < m)
 				jj = idx_j.elem(jji);
 			    }
-			  else if (j < lhs.cols()) 
+			  else if (j < lhs_nc) 
 			    {
-			      for (octave_idx_type i = lhs.cidx(j); 
-				   i < lhs.cidx(j+1); i++)
+			      for (octave_idx_type i = c_lhs.cidx(j); 
+				   i < c_lhs.cidx(j+1); i++)
 				{
-				  stmp.data(kk) = lhs.data(i);
-				  stmp.ridx(kk++) = lhs.ridx(i);
+				  stmp.data(kk) = c_lhs.data(i);
+				  stmp.ridx(kk++) = c_lhs.ridx(i);
 				}
 			    }
 			  stmp.cidx(j+1) = kk;
@@ -2801,10 +2862,6 @@ assign (Sparse<LT>& lhs, const Sparse<RT>& rhs)
 
 	  if (idx_i)
 	    {
-	      // Take a constant copy of lhs. This means that elem won't 
-	      // create missing elements.
-	      const Sparse<LT> c_lhs (lhs);
-
 	      if (rhs_nr == 0 && rhs_nc == 0)
 		lhs.maybe_delete_elements (idx_i);
 	      else if (len == 0)
