@@ -1,11 +1,11 @@
-## Copyright (C) 2005 John W. Eaton
+## Copyright (C) 2005, 2007 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,14 +13,10 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
-## -*- texinfo -*-
-## @deftypefn {Function File} {} __go_draw_axes__ (@var{axis_obj}, @var{plot_stream})
-## Display the axes @var{axis_obj} on @var{plot_stream}.
-## @end deftypefn
+## Undocumented internal function.
 
 ## Author: jwe
 
@@ -39,23 +35,25 @@ function __go_draw_axes__ (h, plot_stream)
 
     if (! isempty (axis_obj.outerposition))
       pos = axis_obj.outerposition;
-      fprintf (plot_stream, "set origin %g, %g;\n", pos(1), pos(2));
-      fprintf (plot_stream, "set size %g, %g;\n", pos(3), pos(4));
+      fprintf (plot_stream, "set origin %.15g, %.15g;\n", pos(1), pos(2));
+      fprintf (plot_stream, "set size %.15g, %.15g;\n", pos(3), pos(4));
     endif
 
     if (! isempty (axis_obj.position))
       pos = axis_obj.position;
-      fprintf (plot_stream, "set origin %g, %g;\n", pos(1), pos(2));
-      fprintf (plot_stream, "set size %g, %g;\n", pos(3), pos(4));
+      fprintf (plot_stream, "set origin %.15g, %.15g;\n", pos(1), pos(2));
+      fprintf (plot_stream, "set size %.15g, %.15g;\n", pos(3), pos(4));
     endif
 
-    if (strcmp (axis_obj.dataaspectratiomode, "manual"))
+    if (strcmpi (axis_obj.dataaspectratiomode, "manual"))
       r = axis_obj.dataaspectratio;
-      fprintf (plot_stream, "set size ratio %g;\n", -r(2)/r(1));
+      fprintf (plot_stream, "set size ratio %.15g;\n", -r(2)/r(1));
     else
       fputs (plot_stream, "set size noratio;\n");
     endif
 
+    fputs (plot_stream, "unset label;\n");
+
     if (! isempty (axis_obj.title))
       t = get (axis_obj.title);
       if (isempty (t.string))
@@ -68,157 +66,133 @@ function __go_draw_axes__ (h, plot_stream)
 
     if (! isempty (axis_obj.xlabel))
       t = get (axis_obj.xlabel);
+      angle = t.rotation;
       if (isempty (t.string))
 	fputs (plot_stream, "unset xlabel;\n");
       else
-	fprintf (plot_stream, "set xlabel \"%s\";\n",
+	fprintf (plot_stream, "set xlabel \"%s\"",
 		 undo_string_escapes (t.string));
+	if (have_newer_gnuplot)
+	  ## Rotation of xlabel not yet support by gnuplot as of 4.2, but
+	  ## there is no message about it.
+	  fprintf (plot_stream, " rotate by %f", angle);
+	endif
+	fputs (plot_stream, ";\n");
       endif
     endif
 
     if (! isempty (axis_obj.ylabel))
       t = get (axis_obj.ylabel);
+      angle = t.rotation;
       if (isempty (t.string))
 	fputs (plot_stream, "unset ylabel;\n");
       else
-	fprintf (plot_stream, "set ylabel \"%s\";\n",
+	fprintf (plot_stream, "set ylabel \"%s\"",
 		 undo_string_escapes (t.string));
+	if (have_newer_gnuplot)
+	  fprintf (plot_stream, " rotate by %f;\n", angle);
+	endif
+	fputs (plot_stream, ";\n");
       endif
     endif
 
     if (! isempty (axis_obj.zlabel))
       t = get (axis_obj.zlabel);
+      angle = t.rotation;
       if (isempty (t.string))
 	fputs (plot_stream, "unset zlabel;\n");
       else
-	fprintf (plot_stream, "set zlabel \"%s\";\n",
+	fprintf (plot_stream, "set zlabel \"%s\"",
 		 undo_string_escapes (t.string));
+	if (have_newer_gnuplot)
+	  ## Rotation of zlabel not yet support by gnuplot as of 4.2, but
+	  ## there is no message about it.
+	  fprintf (plot_stream, " rotate by %f;\n", angle);
+	endif
+	fputs (plot_stream, ";\n");
       endif
     endif
 
-    if (strcmp (axis_obj.xgrid, "on"))
-      fputs (plot_stream, "set grid xtics;\n");
+    if (strcmpi (axis_obj.xaxislocation, "top"))
+      xaxisloc = "x2";
+      xaxisloc_using = "x2";
     else
-      fputs (plot_stream, "set grid noxtics;\n");
+      xaxisloc = "x";
+      xaxisloc_using = "x1";
+    endif
+    if (strcmpi (axis_obj.yaxislocation, "right"))
+      yaxisloc = "y2";
+      yaxisloc_using = "y2";
+    else
+      yaxisloc = "y";
+      yaxisloc_using = "y1";
     endif
 
-    if (strcmp (axis_obj.ygrid, "on"))
-      fputs (plot_stream, "set grid ytics;\n");
+    if (strcmpi (axis_obj.xgrid, "on"))
+      fprintf (plot_stream, "set grid %stics;\n", xaxisloc);
     else
-      fputs (plot_stream, "set grid noytics;\n");
+      fprintf (plot_stream, "set grid no%stics;\n", xaxisloc);
     endif
 
-    if (strcmp (axis_obj.zgrid, "on"))
+    if (strcmpi (axis_obj.ygrid, "on"))
+      fprintf (plot_stream, "set grid %stics;\n", yaxisloc);
+    else
+      fprintf (plot_stream, "set grid no%stics;\n", yaxisloc);
+    endif
+
+    if (strcmpi (axis_obj.zgrid, "on"))
       fputs (plot_stream, "set grid ztics;\n");
     else
       fputs (plot_stream, "set grid ztics;\n");
     endif
 
-    if (strcmp (axis_obj.xminorgrid, "on"))
-      fputs (plot_stream, "set mxtics 5;\n");
-      fputs (plot_stream, "set grid mxtics;\n");
+    if (strcmpi (axis_obj.xminorgrid, "on"))
+      fprintf (plot_stream, "set m%stics 5;\n", xaxisloc);
+      fprintf (plot_stream, "set grid m%stics;\n", xaxisloc);
     else
-      fputs (plot_stream, "set grid nomxtics;\n");
+      fprintf (plot_stream, "set grid nom%stics;\n", xaxisloc);
     endif
 
-    if (strcmp (axis_obj.yminorgrid, "on"))
-      fputs (plot_stream, "set mytics 5;\n");
-      fputs (plot_stream, "set grid mytics;\n");
+    if (strcmpi (axis_obj.yminorgrid, "on"))
+      fprintf (plot_stream, "set m%stics 5;\n", yaxisloc);
+      fprintf (plot_stream, "set grid m%stics;\n", yaxisloc);
     else
-      fputs (plot_stream, "set grid nomytics;\n");
+      fprintf (plot_stream, "set grid nom%stics;\n", yaxisloc);
     endif
 
-    if (strcmp (axis_obj.zminorgrid, "on"))
+    if (strcmpi (axis_obj.zminorgrid, "on"))
       fputs (plot_stream, "set mztics 5;\n");
       fputs (plot_stream, "set grid mztics;\n");
     else
       fputs (plot_stream, "set grid nomztics;\n");
     endif
 
-    if (strcmp (axis_obj.xtickmode, "manual"))
-      xtic = axis_obj.xtick;
-      if (isempty (xtic))
-	fputs (plot_stream, "unset xtics;\n");
-      else
-	## FIXME
-      endif
-    else
-      fputs (plot_stream, "set xtics;\n");
-    endif
+    do_tics (axis_obj, plot_stream);
 
-    if (strcmp (axis_obj.ytickmode, "manual"))
-      ytic = axis_obj.ytick;
-      if (isempty (ytic))
-	fputs (plot_stream, "unset ytics;\n");
-      else
-	## FIXME
-      endif
-    else
-      fputs (plot_stream, "set ytics;\n");
-    endif
-
-    if (strcmp (axis_obj.ztickmode, "manual"))
-      ztic = axis_obj.ztick;
-      if (isempty (ztic))
-	fputs (plot_stream, "unset ztics;\n");
-      else
-	## FIXME
-      endif
-    else
-      fputs (plot_stream, "set ztics;\n");
-    endif
-
-    if (strcmp (axis_obj.xticklabelmode, "manual"))
-      ## FIXME -- we should be able to specify the actual tick labels,
-      ## not just the format.
-      xticlabel = axis_obj.xticklabel;
-      fprintf (plot_stream, "set format x \"%s\";\n", xticlabel);
-    else
-      fputs (plot_stream, "set xtics;\n");
-    endif
-
-    if (strcmp (axis_obj.yticklabelmode, "manual"))
-      ## FIXME -- we should be able to specify the actual tick labels,
-      ## not just the format.
-      yticlabel = axis_obj.yticklabel;
-      fprintf (plot_stream, "set format y \"%s\";\n", yticlabel);
-    else
-      fputs (plot_stream, "set ytics;\n");
-    endif
-
-    if (strcmp (axis_obj.zticklabelmode, "manual"))
-      ## FIXME -- we should be able to specify the actual tick labels,
-      ## not just the format.
-      zticlabel = axis_obj.zticklabel;
-      fprintf (plot_stream, "set format z \"%s\";\n", zticlabel);
-    else
-      fputs (plot_stream, "set ztics;\n");
-    endif
-
-    xlogscale = strcmp (axis_obj.xscale, "log");
+    xlogscale = strcmpi (axis_obj.xscale, "log");
     if (xlogscale)
-      fputs (plot_stream, "set logscale x;\n");
+      fprintf (plot_stream, "set logscale %s;\n", xaxisloc);
     else
-      fputs (plot_stream, "unset logscale x;\n");
+      fprintf (plot_stream, "unset logscale %s;\n", xaxisloc);
     endif
 
-    ylogscale = strcmp (axis_obj.yscale, "log");
+    ylogscale = strcmpi (axis_obj.yscale, "log");
     if (ylogscale)
-      fputs (plot_stream, "set logscale y;\n");
+      fprintf (plot_stream, "set logscale %s;\n", yaxisloc);
     else
-      fputs (plot_stream, "unset logscale y;\n");
+      fprintf (plot_stream, "unset logscale %s;\n", yaxisloc);
     endif
 
-    zlogscale = strcmp (axis_obj.zscale, "log");
+    zlogscale = strcmpi (axis_obj.zscale, "log");
     if (zlogscale)
       fputs (plot_stream, "set logscale z;\n");
     else
       fputs (plot_stream, "unset logscale z;\n");
     endif
 
-    xautoscale = strcmp (axis_obj.xlimmode, "auto");
-    yautoscale = strcmp (axis_obj.ylimmode, "auto");
-    zautoscale = strcmp (axis_obj.zlimmode, "auto");
+    xautoscale = strcmpi (axis_obj.xlimmode, "auto");
+    yautoscale = strcmpi (axis_obj.ylimmode, "auto");
+    zautoscale = strcmpi (axis_obj.zlimmode, "auto");
 
     kids = axis_obj.children;
 
@@ -227,9 +201,9 @@ function __go_draw_axes__ (h, plot_stream)
     data = cell ();
     is_image_data = [];
 
-    xminp = yminp = zminp = realmax ();
-    xmax = ymax = zmax = -realmax ();
-    xmin = ymin = zmin = realmax ();
+    xminp = yminp = zminp = Inf;
+    xmax = ymax = zmax = -Inf;
+    xmin = ymin = zmin = Inf;
 
     [view_cmd, view_fcn, view_zoom] = image_viewer ();
     use_gnuplot_for_images = (ischar (view_fcn)
@@ -261,6 +235,7 @@ function __go_draw_axes__ (h, plot_stream)
 
 	    data_idx++;
 	    is_image_data(data_idx) = true;
+	    parametric(data_idx) = false;
 
 	    [y_dim, x_dim] = size (img_data(:,:,1));
 	    if (x_dim > 1)
@@ -312,7 +287,7 @@ function __go_draw_axes__ (h, plot_stream)
 	    endif
 
 	    titlespec{data_idx} = "title \"\"";
-	    usingclause{data_idx} = sprintf ("binary array=%dx%d scan=yx origin=(%g,%g) dx=%g dy=%g using %s",
+	    usingclause{data_idx} = sprintf ("binary array=%dx%d scan=yx origin=(%.15g,%.15g) dx=%.15g dy=%.15g using %s",
 		x_dim, y_dim, x_origin, y_origin, dx, dy, format);
 	    withclause{data_idx} = sprintf ("with %s", imagetype);
 
@@ -323,6 +298,7 @@ function __go_draw_axes__ (h, plot_stream)
 	case "line"
 	  data_idx++;
 	  is_image_data(data_idx) = false;
+	  parametric(data_idx) = true;
 	  if (isempty (obj.keylabel))
 	    titlespec{data_idx} = "title \"\"";
 	  else
@@ -337,7 +313,6 @@ function __go_draw_axes__ (h, plot_stream)
 	    withclause{data_idx} = sprintf ("with %s linetype %d",
 					    style, typ);
 	  endif
-	  parametric(i) = true;
 	  if (! isempty (obj.zdata))
 	    nd = 3;
 	    xdat = obj.xdata(:);
@@ -398,17 +373,16 @@ function __go_draw_axes__ (h, plot_stream)
 		endif
 		data{data_idx} = [xdat, ydat, xlo, xhi, ylo, yhi]';
 		usingclause{data_idx} = "using ($1):($2):($3):($4):($5):($6)";
-		withclause{data_idx} = sprintf ("with xyerrorbars linestyle %d",
-						data_idx);
+		withclause{data_idx} = "with xyerrorbars";
 	      else
+		## Obtain the limits based on the exact x values.
 		if (xautoscale)
 		  [xmin, xmax, xminp] = get_data_limits (xmin, xmax,
-							 xminp, xdat, tx);
+							 xminp, xdat);
 		endif
 		data{data_idx} = [xdat, ydat, ylo, yhi]';
 		usingclause{data_idx} = "using ($1):($2):($3):($4)";
-		withclause{data_idx} = sprintf ("with yerrorbars linestyle %d",
-						data_idx);
+		withclause{data_idx} = "with yerrorbars";
 	      endif
 	    elseif (xerr)
 	      xlo = xdat-xldat;
@@ -423,8 +397,7 @@ function __go_draw_axes__ (h, plot_stream)
 	      endif
 	      data{data_idx} = [xdat, ydat, xlo, xhi]';
 	      usingclause{data_idx} = "using ($1):($2):($3):($4)";
-	      withclause{data_idx} = sprintf ("with xerrorbars linestyle %d",
-					      data_idx);
+	      withclause{data_idx} = "with xerrorbars";
 	    else
 	      if (xautoscale)
 		[xmin, xmax, xminp] = get_data_limits (xmin, xmax, xminp, xdat);
@@ -433,20 +406,174 @@ function __go_draw_axes__ (h, plot_stream)
 		[ymin, ymax, yminp] = get_data_limits (ymin, ymax, yminp, ydat);
 	      endif
 	      data{data_idx} = [xdat, ydat]';
-	      usingclause{data_idx} = "using ($1):($2)";
+	      usingclause{data_idx} = sprintf ("using ($1):($2) axes %s%s",
+					      xaxisloc_using, yaxisloc_using);
 	    endif
 	  endif
 	  if (! (have_newer_gnuplot || isempty (with)))
 	    if (isempty (withclause{data_idx}))
-	      withclause{data_idx} = sprintf("with %s", with);
+	      withclause{data_idx} = sprintf ("with %s", with);
 	    else
-	      withclause{data_idx} = sprintf("%s %s", withclause{data_idx}, with);
+	      withclause{data_idx} = sprintf ("%s %s", withclause{data_idx},
+					      with);
 	    endif
 	  endif
 
+       case "patch"
+	 if (! isempty (obj.zdata))
+           warning ("gnuplot (as of v4.2) supports only 2D patches, ignoring z values")
+	 endif
+	 nd = 2;
+         cmap = parent_figure_obj.colormap;
+         clim = axis_obj.clim;
+	 [nr, nc] = size (obj.xdata);
+
+	 for i = 1 : nc
+	   xcol = obj.xdata(:,i);
+	   ycol = obj.ydata(:,i);
+
+	   if (xautoscale)
+             [xmin, xmax, xminp] = get_data_limits (xmin, xmax, xminp, xcol);
+	   endif
+	   if (yautoscale)
+	     [ymin, ymax, yminp] = get_data_limits (ymin, ymax, yminp, ycol);
+	   endif
+
+	   if (! isnan (xcol) && ! isnan (ycol))
+	     ## Is the patch closed or not
+	     data_idx++;
+	     is_image_data(data_idx) = false;
+	     parametric(data_idx) = false;
+             titlespec{data_idx} = "title \"\"";
+	     usingclause{data_idx} = "";
+             if (isfield (obj, "facecolor") && isfield (obj, "cdata"))
+               if (strncmp (obj.facecolor, "none", 4))
+		 color = [1, 1, 1];
+
+               elseif (strncmp (obj.facecolor, "flat", 4) ||
+		       strncmp (obj.facecolor, "interp", 6))
+		 if (ndims (obj.cdata) == 2 && ... 
+		     ((nr > 3 && size (obj.cdata, 2) == nc) ...
+                      || (size (obj.cdata, 1) > 1 && ...
+			  size (obj.cdata, 2) == nc)))
+		   ccol = obj.cdata (:, i);
+		 elseif (ndims (obj.cdata) == 3)
+		   ccol = permute (obj.cdata (:, i, :), [1, 3, 2]);
+		 else
+		   ccol = obj.cdata;
+		 endif
+		 if (strncmp (obj.facecolor, "flat", 4))
+		   if (numel(ccol) == 3)
+		     color = ccol;
+		   else
+		     r = 1 + round ((size (cmap, 1) - 1) * ...
+				    (ccol - clim(1))/(clim(2) - clim(1)));
+		     r = max (1, min (r, size (cmap, 1)));
+		     color = cmap(r, :);
+		   endif
+		 elseif (strncmp (obj.facecolor, "interp", 6))
+		   warning ("\"interp\" not supported, using 1st entry of cdata")
+		   r = 1 + round ((size (cmap, 1) - 1) * ccol(1));
+		   r = max (1, min (r, size (cmap, 1)));
+		   color = cmap(r,:);
+		 endif
+	       else
+		 color = obj.facecolor;
+	       endif
+             else
+	       color = [0, 1, 0];
+             endif
+
+	     if (have_newer_gnuplot)
+	       withclause{data_idx} = ...
+	       sprintf ("with filledcurve lc rgb \"#%02x%02x%02x\"", ...
+			round (255*color));
+	     else
+	       if (isequal (color, [0,0,0]))
+		 typ = -1;
+	       elseif (isequal (color, [1,0,0]))
+		 typ = 1;
+	       elseif (isequal (color, [0,1,0]))
+		 typ = 2;
+	       elseif (isequal (color, [0,0,1]))
+		 typ = 3;
+	       elseif (isequal (color, [1,0,1]))
+		 typ = 4;
+	       elseif (isequal (color, [0,1,1]))
+		 typ = 5;
+	       elseif (isequal (color, [1,1,1]))
+		 typ = -1;
+	       elseif (isequal (color, [1,1,0]))
+		 typ = 7;
+	       else
+		 typ = -1;
+	       endif
+	       withclause{data_idx} = sprintf ("with filledcurve lt %d", typ);
+	     endif
+	     data{data_idx} = [xcol, ycol]';
+	     usingclause{data_idx} = "using ($1):($2)";
+	   endif
+
+           ## patch outline
+	   data_idx++;
+           is_image_data(data_idx) = false;
+           parametric(data_idx) = false;
+           titlespec{data_idx} = "title \"\"";
+	   usingclause{data_idx} = "";
+           if (isfield (obj, "edgecolor"))
+             if (strncmp (obj.edgecolor, "none", 4))
+               color = [1, 1, 1];
+             elseif (strncmp (obj.edgecolor, "flat", 4))
+               warning ("\"flat\" for edgecolor not supported");
+               color = [0, 0, 0];
+             elseif (strncmp (obj.edgecolor, "interp", 6))
+               warning ("\"interp\" for edgecolor not supported");
+               color = [0, 0, 0];
+             else
+	       color = obj.edgecolor;
+             endif
+           else
+             color = [0, 0, 0];
+           endif
+	   if (have_newer_gnuplot)
+	     withclause{data_idx} = ...
+	     sprintf ("with lines lc rgb \"#%02x%02x%02x\"", ...
+		      round (255*color));
+	   else
+	     if (isequal (color, [0,0,0]))
+	       typ = -1;
+	     elseif (isequal (color, [1,0,0]))
+	       typ = 1;
+	     elseif (isequal (color, [0,1,0]))
+	       typ = 2;
+	     elseif (isequal (color, [0,0,1]))
+	       typ = 3;
+	     elseif (isequal (color, [1,0,1]))
+	       typ = 4;
+	     elseif (isequal (color, [0,1,1]))
+	       typ = 5;
+	     elseif (isequal (color, [1,1,1]))
+	       typ = -1;
+	     elseif (isequal (color, [1,1,0]))
+	       typ = 7;
+	     else
+	       typ = -1;
+	     endif
+	     withclause{data_idx} = sprintf ("with lines lt %d", typ);
+	   endif
+
+	   if (!isnan (xcol) && !isnan (ycol))
+	     data{data_idx} = [[xcol; xcol(1)], [ycol; ycol(1)]]';
+	   else
+	     data{data_idx} = [xcol, ycol]';
+	   endif
+	   usingclause{data_idx} = "using ($1):($2)";
+	 endfor
+
 	case "surface"
 	  data_idx++;
 	  is_image_data(data_idx) = false;
+	  parametric(data_idx) = false;
 	  [style, typ, with] = do_linestyle_command (obj, data_idx, plot_stream);
 	  if (isempty (obj.keylabel))
 	    titlespec{data_idx} = "title \"\"";
@@ -461,7 +588,6 @@ function __go_draw_axes__ (h, plot_stream)
 	    withclause{data_idx} = sprintf ("with %s linetype %d %s",
 					    style, typ, with);
 	  endif
-	  parametric(i) = false;
 	  nd = 3;
 	  xdat = obj.xdata;
 	  ydat = obj.ydata;
@@ -518,21 +644,74 @@ function __go_draw_axes__ (h, plot_stream)
 	  fputs (plot_stream, "set style data lines;\n");
 	  fputs (plot_stream, "set surface;\n");
 	  fputs (plot_stream, "unset contour;\n");
-	  fputs (plot_stream, "set palette defined (0 \"dark-blue\", 1 \"blue\", 2 \"cyan\", 3 \"yellow\", 4 \"red\" , 5 \"dark-red\");\n");
+	  fprintf (plot_stream, "set cbrange [%g:%g];\n", zmin, zmax);
+
+	  if (have_newer_gnuplot)
+	    surf_colormap = parent_figure_obj.colormap;
+	    palette_size = rows (surf_colormap);
+	    fprintf (plot_stream,
+		     "set palette positive color model RGB maxcolors %i;\n",
+		     palette_size);
+	    fprintf (plot_stream,
+		     "set palette file \"-\" binary record=%d using 1:2:3:4;\n",
+		     palette_size);
+	    fwrite (plot_stream, [1:palette_size; surf_colormap'], "float32");
+	  else
+	    fputs (plot_stream, "set palette defined (0 \"dark-blue\", 1 \"blue\", 2 \"cyan\", 3 \"yellow\", 4 \"red\" , 5 \"dark-red\");\n");
+	  endif
 	  fputs (plot_stream, "unset colorbox;\n");
 
 	case "text"
 	  lpos = obj.position;
 	  label = obj.string;
 	  halign = obj.horizontalalignment;
+	  angle = obj.rotation;
+          units = obj.units;
+	  color = obj.color;
+          if (strcmpi (units, "normalized"))
+            units = "graph";
+          else
+            units = "";
+          endif
+	  
+	  if (isnumeric (color))
+	    if (have_newer_gnuplot)
+	      colorspec = sprintf ("textcolor rgb \"#%02x%02x%02x\"",
+		       round (255*color));
+	    else
+	      if (isequal (color, [0,0,0]))
+		typ = -1;
+	      elseif (isequal (color, [1,0,0]))
+		typ = 1;
+	      elseif (isequal (color, [0,1,0]))
+		typ = 2;
+	      elseif (isequal (color, [0,0,1]))
+		typ = 3;
+	      elseif (isequal (color, [1,0,1]))
+		typ = 4;
+	      elseif (isequal (color, [0,1,1]))
+		typ = 5;
+	      elseif (isequal (color, [1,1,1]))
+		typ = -1;
+	      elseif (isequal (color, [1,1,0]))
+		typ = 7;
+	      else
+		typ = -1;
+	      endif
+	      colorspec = sprintf ("textcolor lt %d", typ);
+	    endif
+	  endif
+
 	  if (nd == 3)
-	    fprintf (plot_stream, "set label \"%s\" at %g,%g,%g %s;\n",
-		     undo_string_escapes (label),
-		     lpos(1), lpos(2), lpos(3), halign);
+	    fprintf (plot_stream,
+		     "set label \"%s\" at %s %.15g,%.15g,%.15g %s rotate by %f %s;\n",
+		     undo_string_escapes (label), units,
+		     lpos(1), lpos(2), lpos(3), halign, angle, colorspec);
 	  else
-	    fprintf (plot_stream, "set label \"%s\" at %g,%g %s;\n",
-		     undo_string_escapes (label),
-		     lpos(1), lpos(2), halign);
+	    fprintf (plot_stream,
+		     "set label \"%s\" at %s %.15g,%.15g %s rotate by %f %s;\n",
+		     undo_string_escapes (label), units,
+		     lpos(1), lpos(2), halign, angle, colorspec);
 	  endif
 
 	otherwise
@@ -546,46 +725,55 @@ function __go_draw_axes__ (h, plot_stream)
 
     if (xautoscale && have_data)
       xlim = get_axis_limits (xmin, xmax, xminp, xlogscale);
+      if (isempty (xlim))
+	return;
+      endif
       set (h, "xlim", xlim, "xlimmode", "auto");
     else
       xlim = axis_obj.xlim;
     endif
-    if (strcmp (axis_obj.xdir, "reverse"))
+    if (strcmpi (axis_obj.xdir, "reverse"))
       xdir = "reverse";
     else
       xdir = "noreverse";
     endif
-    fprintf (plot_stream, "set xrange [%g:%g] %s;\n", xlim, xdir);
+    fprintf (plot_stream, "set %srange [%.15e:%.15e] %s;\n", xaxisloc, xlim, xdir);
 
     if (yautoscale && have_data)
       ylim = get_axis_limits (ymin, ymax, yminp, ylogscale);
+      if (isempty (ylim))
+	return;
+      endif
       set (h, "ylim", ylim, "ylimmode", "auto");
     else
       ylim = axis_obj.ylim;
     endif
-    if (strcmp (axis_obj.ydir, "reverse"))
+    if (strcmpi (axis_obj.ydir, "reverse"))
       ydir = "reverse";
     else
       ydir = "noreverse";
     endif
-    fprintf (plot_stream, "set yrange [%g:%g] %s;\n", ylim, ydir);
+    fprintf (plot_stream, "set %srange [%.15e:%.15e] %s;\n", yaxisloc, ylim, ydir);
 
     if (nd == 3)
       if (zautoscale && have_data)
 	zlim = get_axis_limits (zmin, zmax, zminp, zlogscale);
+	if (isempty (zlim))
+	  return;
+	endif
 	set (h, "zlim", zlim, "zlimmode", "auto");
       else
 	zlim = axis_obj.zlim;
       endif
-      if (strcmp (axis_obj.zdir, "reverse"))
+      if (strcmpi (axis_obj.zdir, "reverse"))
 	zdir = "reverse";
       else
 	zdir = "noreverse";
       endif
-      fprintf (plot_stream, "set zrange [%g:%g] %s;\n", zlim, zdir);
+      fprintf (plot_stream, "set zrange [%.15e:%.15e] %s;\n", zlim, zdir);
     endif
 
-    if (strcmp (axis_obj.box, "on"))
+    if (strcmpi (axis_obj.box, "on"))
       if (nd == 3)
 	fputs (plot_stream, "set border 4095;\n");
       else
@@ -600,16 +788,25 @@ function __go_draw_axes__ (h, plot_stream)
       endif
     endif
 
-    if (strcmp (axis_obj.key, "on"))
-      if (strcmp (axis_obj.keybox, "on"))
+    if (strcmpi (axis_obj.key, "on"))
+      if (strcmpi (axis_obj.keybox, "on"))
 	box = "box";
       else
 	box = "nobox";
       endif
       inout = "inside";
-      switch (axis_obj.keypos)
+      keypos = axis_obj.keypos;
+      if (ischar (keypos))
+	keypos = lower (keypos);
+	keyout = findstr (keypos, "outside");
+	if (! isempty (keyout))
+	  inout = "outside";
+	  keypos = keypos (1:keyout-1);
+	endif
+      endif
+      switch (keypos)
 	case -1
-	  pos = "right bottom";
+	  pos = "right top";
 	  inout = "outside";
 	case 1
 	  pos = "right top";
@@ -617,8 +814,29 @@ function __go_draw_axes__ (h, plot_stream)
 	  pos = "left top";
 	case 3
 	  pos = "left bottom";
-	case 4
+	case {4, 0}
 	  pos = "right bottom";
+	case "north"
+	  pos = "center top";
+	case "south"
+	  pos = "center bottom";
+	case "east"
+	  pos = "right center";
+	case "west"
+	  pos = "left center";
+	case "northeast"
+	  pos = "right top";
+	case "northwest"
+	  pos = "left top";
+	case "southeast"
+	  pos = "right bottom";
+	case "southwest"
+	  pos = "left bottom";
+	case "best" 
+	  pos = "";
+	  warning ("legend: 'Best' not yet implemented for location specifier.\n");
+	  ## least conflict with data in plot
+	  ## least unused space outside plot
 	otherwise
 	  pos = "";
       endswitch
@@ -650,7 +868,7 @@ function __go_draw_axes__ (h, plot_stream)
 	  rot_z += 360;
 	endwhile
  	fputs (plot_stream, "set ticslevel 0;\n");
-	fprintf (plot_stream, "set view %g, %g;\n", rot_x, rot_z);
+	fprintf (plot_stream, "set view %.15g, %.15g;\n", rot_x, rot_z);
       endif
       fprintf (plot_stream, "%s \"-\" %s %s %s \\\n", plot_cmd,
 	       usingclause{1}, titlespec{1}, withclause{1});
@@ -684,9 +902,14 @@ function [xmin, xmax, xminp] = get_data_limits (xmin, xmax, xminp, xdat, tx)
   xmax = max (xmax, max (xdat));
   if (nargin == 5)
     tx = tx(! isinf (xdat) & tx > 0);
-    xminp = min (xminp, min (tx));
+    if (! isempty (tx))
+      xminp = min (xminp, min (tx));
+    endif
   else
-    xminp = min (xminp, min (xdat(xdat>0)));
+    tmp = min (xdat(xdat > 0));
+    if (! isempty (tmp))
+      xminp = min (xminp, tmp);
+    endif
   endif
 endfunction
 
@@ -698,14 +921,13 @@ function lim = get_axis_limits (min_val, max_val, min_pos, logscale)
 
   if (logscale)
     if (isinf (min_pos))
+      lim = [];
       warning ("axis: logscale with no positive values to plot");
+      return;
     endif
     if (min_val <= 0)
-      min_val = min_pos;
-      if (max_val <= 0)
-	max_val = min_pos;
-      endif
       warning ("axis: omitting nonpositive data in log plot");
+      min_val = min_pos;
     endif
     ## FIXME -- maybe this test should also be relative?
     if (abs (min_val - max_val) < sqrt (eps))
@@ -793,10 +1015,21 @@ function [style, typ, with] = do_linestyle_command (obj, idx, plot_stream)
       otherwise
 	lt = "";
     endswitch
-    if (! isempty (lt))
-      fprintf (plot_stream, " linetype %s", lt);
-      found_style = true;
-    endif
+
+    ## FIXME -- linetype is currently broken, since it disables the
+    ## gnuplot default dashed and solid linestyles with the only
+    ## benefit of being able to specify '--' and get a single sized
+    ## dashed line of identical dash pattern for all called this way.
+    ## All dash patterns are a subset of "with lines" and none of the
+    ## lt specifications will correctly propagate into the x11 terminal
+    ## or the print command.   Therefore, it is currently disabled in
+    ## order to allow print (..., "-dashed") etc. to work correctly.
+
+    ##    if (! isempty (lt))
+    ##      fprintf (plot_stream, " linetype %s", lt);
+    ##      found_style = true;
+    ##    endif
+
   else
     lt = "";
   endif
@@ -868,7 +1101,7 @@ function [style, typ, with] = do_linestyle_command (obj, idx, plot_stream)
     if (have_newer_gnuplot)
       fprintf (plot_stream, " pointsize %f", obj.markersize);
     else
-      if (! strcmp (style, "lines"))
+      if (! strcmpi (style, "lines"))
 	with = sprintf ("%s ps %f", with, obj.markersize);
       endif
     endif
@@ -898,7 +1131,7 @@ function __gnuplot_write_data__ (plot_stream, data, nd, parametric)
 
   if (nd == 2)
     nan_elts = find (sum (isnan (data)));
-    fmt = strcat (repmat ("%g ", 1, rows (data)), "\n");
+    fmt = strcat (repmat ("%.15g ", 1, rows (data)), "\n");
     if (isempty (nan_elts))
       fprintf (plot_stream, fmt, data);
     else
@@ -918,11 +1151,11 @@ function __gnuplot_write_data__ (plot_stream, data, nd, parametric)
   else
     ## FIXME -- handle NaNs here too?
     if (parametric)
-      fprintf (plot_stream, "%g %g %g\n", data);
+      fprintf (plot_stream, "%.15g %.15g %.15g\n", data);
     else
       nc = columns (data);
       for j = 1:3:nc
-	fprintf (plot_stream, "%g %g %g\n", data(:,j:j+2)');
+	fprintf (plot_stream, "%.15g %.15g %.15g\n", data(:,j:j+2)');
 	fputs (plot_stream, "\n");
       endfor
     endif
@@ -930,3 +1163,69 @@ function __gnuplot_write_data__ (plot_stream, data, nd, parametric)
   fputs (plot_stream, "e\n");
 
 endfunction
+
+function do_tics (obj, plot_stream)
+  if (strcmpi (obj.xaxislocation, "top"))
+    do_tics_1 (obj.xtickmode, obj.xtick, obj.xticklabelmode, obj.xticklabel,
+	       "x2", plot_stream);
+    do_tics_1 ("manual", [], obj.xticklabelmode, obj.xticklabel,
+	       "x", plot_stream);
+  else
+    do_tics_1 (obj.xtickmode, obj.xtick, obj.xticklabelmode, obj.xticklabel,
+	       "x", plot_stream);
+    do_tics_1 ("manual", [], obj.xticklabelmode, obj.xticklabel,
+	       "x2", plot_stream);
+  endif
+  if (strcmpi (obj.yaxislocation, "right"))
+    do_tics_1 (obj.ytickmode, obj.ytick, obj.yticklabelmode, obj.yticklabel,
+	       "y2", plot_stream);
+    do_tics_1 ("manual", [], obj.yticklabelmode, obj.yticklabel,
+	       "y", plot_stream);
+  else
+    do_tics_1 (obj.ytickmode, obj.ytick, obj.yticklabelmode, obj.yticklabel,
+	       "y", plot_stream);
+    do_tics_1 ("manual", [], obj.yticklabelmode, obj.yticklabel,
+	       "y2", plot_stream);
+  endif
+  do_tics_1 (obj.ztickmode, obj.ztick, obj.zticklabelmode, obj.zticklabel,
+	     "z", plot_stream);
+endfunction
+
+function do_tics_1 (ticmode, tics, labelmode, labels, ax, plot_stream)
+  if (strcmpi (ticmode, "manual"))
+    if (isempty (tics))
+      fprintf (plot_stream, "unset %stics;\n", ax);
+    elseif (strcmpi (labelmode, "manual") && ! isempty (labels))
+      if (ischar (labels))
+	labels = cellstr (labels);
+      endif
+      if (iscellstr (labels))
+	k = 1;
+	ntics = numel (tics);
+	nlabels = numel (labels);
+	fprintf (plot_stream, "set format %s \"%%s\";\n", ax);
+	fprintf (plot_stream, "set %stics (", ax);
+	for i = 1:ntics
+	  fprintf (plot_stream, " \"%s\" %g", labels(k++), tics(i))
+	  if (i < ntics)
+	    fputs (plot_stream, ", ");
+	  endif
+	  if (k > nlabels)
+	    k = 1;
+	  endif
+	endfor
+	fputs (plot_stream, ");\n");
+      else
+	error ("unsupported type of ticklabel");
+      endif
+    else
+      fprintf (plot_stream, "set format %s \"%%g\";\n", ax);
+      fprintf (plot_stream, "set %stics (", ax);
+      fprintf (plot_stream, " %g,", tics(1:end-1));
+      fprintf (plot_stream, " %g);\n", tics(end));
+    endif
+  else
+    fprintf (plot_stream, "set format %s \"%%g\";\n", ax);
+    fprintf (plot_stream, "set %stics;\n", ax);
+  endif
+endfunction
