@@ -1,11 +1,11 @@
-## Copyright (C) 2000, 2001, 2002 Teemu Ikonen
+## Copyright (C) 2000, 2001, 2002, 2004, 2005, 2006, 2007 Teemu Ikonen
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,8 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} errorbar (@var{args})
@@ -35,7 +34,7 @@
 ## If more than two arguments are given, they are interpreted as
 ##
 ## @example
-## errorbar (@var{x}, @var{y}, ..., @var{fmt}, ...)
+## errorbar (@var{x}, @var{y}, @dots{}, @var{fmt}, @dots{})
 ## @end example
 ##
 ## @noindent
@@ -49,11 +48,11 @@
 ## corresponding columns of @var{x} and errorbars are drawn from
 ## the corresponding columns of error parameters.
 ##
-## If @var{fmt} is missing, yerrorbars ("~") plot style
-## is assumed.
+## If @var{fmt} is missing, yerrorbars ("~") plot style is assumed.
+##
 ## If the @var{fmt} argument is supplied, it is interpreted as in
-## normal plots (See __pltopt__). In addition the following
-## plot styles are supported by errorbar:
+## normal plots. In addition the following plot styles are supported by
+## errorbar:
 ##
 ## @table @samp
 ## @item ~
@@ -73,36 +72,36 @@
 ##
 ## @item #~>
 ## Set boxxyerrorbars plot style.
-##
 ## @end table
 ##
 ## Examples:
 ##
 ## @example
-## errorbar(@var{x}, @var{y}, @var{ex}, ">")
+## errorbar (@var{x}, @var{y}, @var{ex}, ">")
 ## @end example
 ##
-## xerrorbar plot of @var{y} versus @var{x} with @var{x} errorbars drawn from
-## @var{x}-@var{ex} to @var{x}+@var{ex}.
+## produces an xerrorbar plot of @var{y} versus @var{x} with @var{x}
+## errorbars drawn from @var{x}-@var{ex} to @var{x}+@var{ex}.
 ##
 ## @example
-## errorbar(@var{x}, @var{y1}, @var{ey}, "~", @var{x}, @var{y2}, @var{ly}, @var{uy})
+## errorbar (@var{x}, @var{y1}, @var{ey}, "~",
+##           @var{x}, @var{y2}, @var{ly}, @var{uy})
 ## @end example
 ##
-## Two yerrorbar plots with @var{y1} and @var{y2} versus @var{x}.
+## produces yerrorbar plots with @var{y1} and @var{y2} versus @var{x}.
 ## Errorbars for @var{y1} are drawn from @var{y1}-@var{ey} to
 ## @var{y1}+@var{ey}, errorbars for @var{y2} from @var{y2}-@var{ly} to
 ## @var{y2}+@var{uy}.
 ##
 ## @example
-## errorbar(@var{x}, @var{y}, @var{lx}, @var{ux}, @var{ly}, @var{uy}, "~>")
+## errorbar (@var{x}, @var{y}, @var{lx}, @var{ux},
+##           @var{ly}, @var{uy}, "~>")
 ## @end example
 ##
-## xyerrorbar plot of @var{y} versus @var{x} where @var{x} errorbars
-## are drawn from @var{x}-@var{lx} to @var{x}+@var{ux} and @var{y} errorbars
-## from @var{y}-@var{ly} to @var{y}+@var{uy}.
-## @seealso{semilogx, semilogy, loglog, polar, mesh, contour, __pltopt__,
-## bar, stairs, xlabel, ylabel, title}
+## produces an xyerrorbar plot of @var{y} versus @var{x} in which
+## @var{x} errorbars are drawn from @var{x}-@var{lx} to @var{x}+@var{ux}
+## and @var{y} errorbars from @var{y}-@var{ly} to @var{y}+@var{uy}.
+## @seealso{semilogxerr, semilogyerr, loglogerr}
 ## @end deftypefn
 
 ## Created: 18.7.2000
