@@ -1,11 +1,12 @@
-## Copyright (C) 1996, 1997 John W. Eaton
+## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2004,
+##               2005, 2006, 2007 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,14 +14,17 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} plot (@var{args})
-## This function produces two-dimensional plots.  Many different
-## combinations of arguments are possible.  The simplest form is
+## @deftypefn {Function File} {} plot (@var{y})
+## @deftypefnx {Function File} {} plot (@var{x}, @var{y})
+## @deftypefnx {Function File} {} plot (@var{x}, @var{y}, @var{property}, @var{value}, @dots{})
+## @deftypefnx {Function File} {} plot (@var{x}, @var{y}, @var{fmt})
+## @deftypefnx {Function File} {} plot (@var{h}, @dots{})
+## Produces two-dimensional plots.  Many different combinations of
+## arguments are possible.  The simplest form is
 ##
 ## @example
 ## plot (@var{y})
@@ -37,20 +41,26 @@
 ## If more than one argument is given, they are interpreted as
 ##
 ## @example
-## plot (@var{x}, @var{y}, @var{fmt}, @dots{})
+## plot (@var{y}, @var{property}, @var{value}, @dots{})
 ## @end example
 ##
 ## @noindent
-## or as
+## or
 ##
 ## @example
 ## plot (@var{x}, @var{y}, @var{property}, @var{value}, @dots{})
 ## @end example
 ##
 ## @noindent
-## where @var{y}, @var{fmt}, @var{property} and @var{value} are optional,
-## and any number of argument sets may appear.  The @var{x} and @var{y} 
-## values are interpreted as follows:
+## or
+##
+## @example
+## plot (@var{x}, @var{y}, @var{fmt}, @dots{})
+## @end example
+##
+## @noindent
+## and so on.  Any number of argument sets may appear.  The @var{x} and
+## @var{y} values are interpreted as follows:
 ##
 ## @itemize @bullet
 ## @item
@@ -59,7 +69,7 @@
 ## the elements, starting with 1.
 ##
 ## @item
-## If the @var{x} is a vector and @var{y} is a matrix, the
+## If the @var{x} is a vector and @var{y} is a matrix, then
 ## the columns (or rows) of @var{y} are plotted versus @var{x}.
 ## (using whichever combination matches, with columns tried first.)
 ##
@@ -81,6 +91,10 @@
 ## If both arguments are scalars, a single point is plotted.
 ## @end itemize
 ##
+## Multiple property-value pairs may be specified, but they must appear
+## in pairs.  These arguments are applied to the lines drawn by
+## @code{plot}.
+##
 ## If the @var{fmt} argument is supplied, it is interpreted as
 ## follows.  If @var{fmt} is missing, the default gnuplot line style
 ## is assumed.
@@ -123,28 +137,12 @@
 ## style.
 ## @end table
 ##
-## The color line styles have the following meanings on terminals that
-## support color.
-##
-## @example
-## Number  Gnuplot colors  (lines)points style
-##   1       red                   *
-##   2       green                 +
-##   3       blue                  o
-##   4       magenta               x
-##   5       cyan                house
-##   6       brown            there exists
-## @end example
-##
-## The @var{fmt} argument can also be used to assign key titles.
+## The @var{fmt} argument may also be used to assign key titles.
 ## To do so, include the desired title between semi-colons after the
 ## formatting sequence described above, e.g. "+3;Key Title;"
 ## Note that the last semi-colon is required and will generate an error if
 ## it is left out.
 ##
-## If a @var{property} is given it must be followed by @var{value}.  The
-## property value pairs are applied to the lines drawn by @code{plot}.
-##
 ## Here are some plot examples:
 ##
 ## @example
@@ -159,7 +157,7 @@
 ## plot (b, "*", "markersize", 3)
 ## @end example
 ##
-## This command will plot the data in the variable @code{b} will be plotted
+## This command will plot the data in the variable @code{b},
 ## with points displayed as @samp{*} with a marker size of 3.
 ##
 ## @example
@@ -169,17 +167,34 @@
 ##
 ## This will plot the cosine and sine functions and label them accordingly
 ## in the key.
-## @seealso{semilogx, semilogy, loglog, polar, mesh, contour, __pltopt__
-## bar, stairs, errorbar, xlabel, ylabel, title, print}
+##
+## If the first argument is an axis handle, then plot into these axes, 
+## rather than the current axis handle returned by @code{gca}. 
+## @seealso{semilogx, semilogy, loglog, polar, mesh, contour, bar,
+## stairs, errorbar, xlabel, ylabel, title, print}
 ## @end deftypefn
 
 ## Author: jwe
 
 function retval = plot (varargin)
 
-  newplot ();
-
-  tmp = __plt__ ("plot", gca (), varargin{:});
+  if (isscalar (varargin{1}) && ishandle (varargin{1}))
+    h = varargin {1};
+    if (! strcmp (get (h, "type"), "axes"))
+      error ("plot: expecting first argument to be an axes object");
+    endif
+    oldh = gca ();
+    unwind_protect
+      axes (h);
+      newplot ();
+      tmp = __plt__ ("plot", h, varargin{2:end});
+    unwind_protect_cleanup
+      axes (oldh);
+    end_unwind_protect
+  else
+    newplot ();
+    tmp = __plt__ ("plot", gca (), varargin{:});
+  endif
 
   if (nargout > 0)
     retval = tmp;
