@@ -1,11 +1,12 @@
-## Copyright (C) 1996 Kurt Hornik
+## Copyright (C) 1996, 1998, 1999, 2000, 2002, 2004, 2005, 2006, 2007
+##               Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,35 +14,39 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} deblank (@var{s})
-## Removes the trailing blanks and nulls from the string @var{s}.
-## If @var{s} is a matrix, @var{deblank} trims each row to the 
-## length of longest string.
+## Remove trailing blanks and nulls from @var{s}.  If @var{s}
+## is a matrix, @var{deblank} trims each row to the length of longest
+## string.  If @var{s} is a cell array, operate recursively on each
+## element of the cell array.
 ## @end deftypefn
 
-## Author: Kurt Hornik <Kurt.Hornik@ci.tuwien.ac.at>
+## Author: Kurt Hornik <Kurt.Hornik@wu-wien.ac.at>
 ## Adapted-By: jwe
 
-function t = deblank (s)
+function s = deblank (s)
 
   if (nargin != 1)
-    usage ("deblank (s)");
+    print_usage ();
   endif
 
-  if (isstr (s))
+  if (ischar (s))
 
     k = find (! isspace (s) & s != "\0");
     if (isempty (s) || isempty (k))
-      t = "";
+      s = "";
     else
-      t = s(:,1:ceil (max (k) / rows (s)));
+      s = s(:,1:ceil (max (k) / rows (s)));
     endif
 
+  elseif (iscell(s))
+
+    s = cellfun (@deblank, s, "UniformOutput", false);
+
   else
     error ("deblank: expecting string argument");
   endif
