@@ -1,19 +1,20 @@
-## Copyright (C) 2005 Paul Kienzle
+## Copyright (C) 2005, 2006, 2007 Paul Kienzle
 ##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2 of the License, or
-## (at your option) any later version.
+## This file is part of Octave.
 ##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
+## Octave is free software; you can redistribute it and/or modify it
+## under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
+##
+## Octave is distributed in the hope that it will be useful, but
+## WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301  USA
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} test @var{name}
@@ -53,13 +54,13 @@
 ## the current test.
 ##
 ## Called with a single output argument @var{success}, @code{test} returns
-## true is all of the tests were successful. Called with two output arguments
-## @var{n} and @var{max}, the number of sucessful test and the total number
+## true if all of the tests were successful. Called with two output arguments
+## @var{n} and @var{max}, the number of successful tests and the total number
 ## of tests in the file @var{name} are returned.
 ##
 ## If the second argument is the string 'grabdemo', the contents of the demo
 ## blocks are extracted but not executed. Code for all code blocks is
-## concatented and returned as @var{code} with @var{idx} being a vector of
+## concatenated and returned as @var{code} with @var{idx} being a vector of
 ## positions of the ends of the demo blocks.
 ##
 ## If the second argument is 'explain', then @var{name} is ignored and an
@@ -74,13 +75,15 @@
 
 ## PKG_ADD: mark_as_command test
 
-function [__ret1, __ret2] = test (__name, __flag, __fid)
+function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
   ## information from test will be introduced by "key" 
   persistent __signal_fail =  "!!!!! ";
   persistent __signal_empty = "????? ";
   persistent __signal_block = "  ***** ";
   persistent __signal_file =  ">>>>> ";
 
+  __xfail = 0;
+
   if (nargin < 2 || isempty (__flag))
     __flag = "quiet";
   endif
@@ -299,7 +302,7 @@ function [__ret1, __ret2] = test (__name, __flag, __fid)
       endif
       
       ## strip comments off the variables
-      __idx = find (__vars=="%" | __vars == "#");
+      __idx = find (__vars == "%" | __vars == "#");
       if (! isempty (__idx))
 	__vars = __vars(1:__idx(1)-1);
       endif
@@ -412,7 +415,7 @@ function [__ret1, __ret2] = test (__name, __flag, __fid)
       __code = ""; # code already processed
       
     ## TEST
-    elseif (strcmp (__type, "test"))
+    elseif (strcmp (__type, "test") || strcmp (__type, "xtest"))
       __istest = 1;
       ## code will be evaluated below
       
@@ -436,8 +439,13 @@ function [__ret1, __ret2] = test (__name, __flag, __fid)
 		       __shared_r,__shared, __code));
 	eval (sprintf ("%s__test__(%s);", __shared_r, __shared));
       catch
-	__success = 0;
-	__msg = sprintf ("%stest failed\n%s", __signal_fail, __error_text__);
+        if (strcmp (__type, "xtest"))
+           __msg = sprintf ("%sknown failure\n%s", __signal_fail, __error_text__);
+	   __xfail++;
+        else
+           __msg = sprintf ("%stest failed\n%s", __signal_fail, __error_text__);
+	   __success = 0;
+        endif
 	if (isempty (__error_text__))
 	  error ("empty error text, probably Ctrl-C --- aborting"); 
 	endif
@@ -480,7 +488,12 @@ function [__ret1, __ret2] = test (__name, __flag, __fid)
   eval (__clear, "");
 
   if (nargout == 0)
-    printf ("PASSES %d out of %d tests\n", __successes, __tests);
+    if (__xfail)
+      printf ("PASSES %d out of %d tests (%d expected failures)\n",
+	      __successes, __tests, __xfail);
+    else
+      printf ("PASSES %d out of %d tests\n", __successes, __tests);
+    endif
   elseif (__grabdemo)
     __ret1 = __demo_code;
     __ret2 = __demo_idx;
@@ -489,6 +502,7 @@ function [__ret1, __ret2] = test (__name, __flag, __fid)
   else
     __ret1 = __successes;
     __ret2 = __tests;
+    __ret3 = __xfail;
   endif
 endfunction
 
@@ -579,6 +593,9 @@ function body = __extract_test_code (nm)
   endif
 endfunction
 
+### Test for a known failure
+%!xtest error("This test is known to fail")
+
 ### example from toeplitz
 %!shared msg
 %! msg="expecting vector arguments";
