@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 2000, 2001, 2004, 2005 Gabriele Pannocchia
+Copyright (C) 2000, 2001, 2004, 2005, 2006, 2007 Gabriele Pannocchia
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -39,12 +38,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "pr-output.h"
 #include "utils.h"
 
-static inline double
-ABS (double x)
-{
-  return x < 0 ? -x : x;
-}
-
 static Matrix
 null (const Matrix& A, octave_idx_type& rank)
 {
@@ -81,6 +74,10 @@ null (const Matrix& A, octave_idx_type& rank)
 	retval = V.extract (0, rank, A_nc-1, A_nc-1);
       else
 	retval.resize (A_nc, 0);
+
+      for (octave_idx_type i = 0; i < retval.numel (); i++)
+	if (std::abs (retval(i)) < DBL_EPSILON)
+	  retval(i) = 0;
     }
 
   return retval;
@@ -125,14 +122,14 @@ qp (const Matrix& H, const ColumnVector& q,
 
       for (octave_idx_type i = 0; i < n_in; i++)
 	{
-	  res(i) /= (1.0 + ABS (bin(i)));
+	  res(i) /= (1.0 + std::abs (bin(i)));
 
 	  if (res(i) < rtol)
 	    {
 	      n_act++;
 	      Aact = Aact.stack (Ain.row (i));
 	      bact.resize (n_act, bin(i));
-	      Wact.resize (n_act, i);
+	      Wact.resize (n_act-n_eq, i);
 	    }
 	}
     }
@@ -226,12 +223,9 @@ qp (const Matrix& H, const ColumnVector& q,
 
 	  octave_idx_type dimZ = n - rank;
 
-	  // XXX FIXME XXX -- still remain to handle the case of
+	  // FIXME -- still remain to handle the case of
 	  // non-full rank active set matrix.
 
-	  // Computing the Y matrix (orthogonal to Z)
-	  Y = Aact.pseudo_inverse ();
-
 	  // Reduced Hessian
 	  Matrix Zt = Z.transpose ();
 	  Matrix rH = Zt * H * Z;
@@ -304,7 +298,7 @@ qp (const Matrix& H, const ColumnVector& q,
       // Checking the step-size.
       ColumnVector abs_p (n);
       for (octave_idx_type i = 0; i < n; i++)
-	abs_p(i) = ABS (p(i));
+	abs_p(i) = std::abs (p(i));
       double max_p = abs_p.max ();
 
       if (max_p < rtol)
@@ -319,16 +313,10 @@ qp (const Matrix& H, const ColumnVector& q,
 	      // Computing the multipliers only for the inequality
 	      // constraints that are active.  We do NOT compute
 	      // multipliers for the equality constraints.
+ 	      Y = Aact.pseudo_inverse ();
 	      Matrix Yt = Y.transpose ();
 	      Yt = Yt.extract_n (n_eq, 0, n_act-n_eq, n);
 	      lambda_tmp = Yt * (g + H * p);
-	      if (n_act - n_eq < n_in)
-		{
-		  lambda_tmp.resize (n_in, 0.0);
-
-		  for (octave_idx_type i = n_act-n_eq; i < n_in; i++)
-		    lambda_tmp(i) = 0;
-		}
 
 	      // Checking the multipliers.  We remove the most
 	      // negative from the set (if any).
@@ -391,7 +379,7 @@ qp (const Matrix& H, const ColumnVector& q,
 
 		  for (octave_idx_type j = 0; j < n_act-n_eq; j++)
 		    {
-		      if (Wact(j) == i)
+		      if (Wact(j) == i - n_eq)
 			{
 			  found = true;
 			  break;
@@ -429,7 +417,7 @@ qp (const Matrix& H, const ColumnVector& q,
 		  n_act++;
 		  Aact = Aact.stack (Ain.row (is_block));
 		  bact.resize (n_act, bin(is_block));
-		  Wact.resize (n_act, is_block);
+		  Wact.resize (n_act-n_eq, is_block);
 
 		  // Adding the reduced step
 		  x += alpha * p;
@@ -476,7 +464,10 @@ qp (const Matrix& H, const ColumnVector& q,
 }
 
 DEFUN_DLD (__qp__, args, ,
-  "[x, lambda, info, iter] = __qp__ (x0, H, q, Aeq, beq, Ain, bin, maxit)")
+  "-*- texinfo -*-\n\
+@deftypefn {Loadable Function} {[@var{x}, @var{lambda}, @var{info}, @var{iter}] =} __qp__ (@var{x0}, @var{H}, @var{q}, @var{Aeq}, @var{beq}, @var{Ain}, @var{bin}, @var{maxit})\n\
+Undocumented internal function.\n\
+@end deftypefn")
 {
   octave_value_list retval;
 
@@ -512,7 +503,7 @@ DEFUN_DLD (__qp__, args, ,
 	error ("__qp__: invalid arguments");
     }
   else
-    print_usage ("__qp__");
+    print_usage ();
 
   return retval;
 }
