@@ -1,12 +1,14 @@
 /*
 
-Copyright (C) 2004 David Bateman
-Copyright (C) 1998-2004 Andy Adler
+Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
+
+This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -73,7 +74,7 @@ is forced back to a full matrix is resulting matrix is sparse\n\
 Create a sparse matrix given integer index vectors @var{i} and @var{j},\n\
 a 1-by-@code{nnz} vector of real of complex values @var{sv}, overall\n\
 dimensions @var{m} and @var{n} of the sparse matrix.  The argument\n\
-@code{nzmax} is ignored but accepted for compatability with @sc{Matlab}.\n\
+@code{nzmax} is ignored but accepted for compatibility with @sc{Matlab}.\n\
 \n\
 @strong{Note}: if multiple values are specified with the same\n\
 @var{i}, @var{j} indices, the corresponding values in @var{s} will\n\
@@ -1026,6 +1027,21 @@ make_spdiag (const octave_value& a, const octave_value& b)
   return retval;
 }
 
+static octave_value
+make_spdiag (const octave_value& a)
+{
+  octave_value retval;
+  octave_idx_type nr = a.rows ();
+  octave_idx_type nc = a.columns ();
+
+  if (nr == 0 || nc == 0)
+    retval = SparseMatrix ();
+  else
+    retval = make_spdiag (a, octave_value (0.));
+
+  return retval;
+}
+
 // PKG_ADD: dispatch ("diag", "spdiag", "sparse matrix");
 // PKG_ADD: dispatch ("diag", "spdiag", "sparse complex matrix");
 // PKG_ADD: dispatch ("diag", "spdiag", "sparse bool matrix");
@@ -1039,6 +1055,7 @@ on the @var{-k}-th sub-diagonal.  The default value of @var{k} is 0, and\n\
 the vector is placed on the main diagonal.  For example,\n\
 \n\
 @example\n\
+@group\n\
 spdiag ([1, 2, 3], 1)\n\
 ans =\n\
 \n\
@@ -1046,7 +1063,12 @@ Compressed Column Sparse (rows=4, cols=4, nnz=3)\n\
   (1 , 2) -> 1\n\
   (2 , 3) -> 2\n\
   (3 , 4) -> 3\n\
+@end group\n\
 @end example\n\
+\n\
+@noindent\n\
+Given a matrix argument, instead of a vector, @code{spdiag} extracts the\n\
+@var{k}-th diagonal of the sparse matrix.\n\
 @seealso{diag}\n\
 @end deftypefn")
 {
@@ -1055,7 +1077,7 @@ Compressed Column Sparse (rows=4, cols=4, nnz=3)\n\
   int nargin = args.length ();
 
   if (nargin == 1 && args(0).is_defined ())
-    retval = make_spdiag (args(0), octave_value(0.));
+    retval = make_spdiag (args(0));
   else if (nargin == 2 && args(0).is_defined () && args(1).is_defined ())
     retval = make_spdiag (args(0), args(1));
   else
