@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+              2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -30,6 +30,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <iomanip>
 #include <fstream>
+#include <sstream>
 #include <string>
 
 #include <Array.h>
@@ -41,7 +42,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "byte-swap.h"
 #include "lo-ieee.h"
 #include "lo-mappers.h"
-#include "lo-sstream.h"
 #include "lo-utils.h"
 #include "str-vec.h"
 #include "quit.h"
@@ -70,7 +70,7 @@ convert_to_valid_int (const octave_value& tc, int& conv_err)
 
   if (! error_state)
     {
-      if (! lo_ieee_is_NaN_or_NA (dval))
+      if (! lo_ieee_isnan (dval))
 	{
 	  int ival = NINT (dval);
 
@@ -93,7 +93,7 @@ get_size (double d, const std::string& who)
 {
   int retval = -1;
 
-  if (! lo_ieee_is_NaN_or_NA (d))
+  if (! lo_ieee_isnan (d))
     {
       if (! xisinf (d))
 	{
@@ -176,7 +176,7 @@ scanf_format_list::scanf_format_list (const std::string& s)
       have_more = true;
 
       if (! buf)
-	buf = new OSSTREAM ();
+	buf = new std::ostringstream ();
 
       if (s[i] == '%')
 	{
@@ -251,11 +251,7 @@ scanf_format_list::add_elt_to_list (int width, bool discard, char type,
 {
   if (buf)
     {
-      *buf << OSSTREAM_ENDS;
-
-      std::string text = OSSTREAM_STR (*buf);
-
-      OSSTREAM_FREEZE (*buf);
+      std::string text = buf->str ();
 
       if (! text.empty ())
 	{
@@ -288,7 +284,7 @@ expand_char_class (const std::string& s)
       unsigned char c = s[i++];
 
       if (c == '-' && i > 1 && i < len
-	  && (unsigned char) s[i-2] <= (unsigned char) s[i])
+	  && static_cast<unsigned char> (s[i-2]) <= static_cast<unsigned char> (s[i]))
 	{
 	  // Add all characters from the range except the first (we
 	  // already added it below).
@@ -612,7 +608,7 @@ printf_format_list::printf_format_list (const std::string& s)
 
 	  if (! buf)
 	    {
-	      buf = new OSSTREAM ();
+	      buf = new std::ostringstream ();
 	      empty_buf = true;
 	    }
 
@@ -681,11 +677,7 @@ printf_format_list::add_elt_to_list (int args, const std::string& flags,
 {
   if (buf)
     {
-      *buf << OSSTREAM_ENDS;
-
-      std::string text = OSSTREAM_STR (*buf);
-
-      OSSTREAM_FREEZE (*buf);
+      std::string text = buf->str ();
 
       if (! text.empty ())
 	{
@@ -905,10 +897,11 @@ octave_base_stream::file_number (void)
   // std::filebuf (nor in the GNU libstdc++-v3 implementation). We cache
   // the descriptor in c_file_ptr_buf, and then extract it here.
 
-  c_file_ptr_buf *ibuf = is ?
-    dynamic_cast<c_file_ptr_buf *> (is->rdbuf ()) : 0;
-  c_file_ptr_buf *obuf = os ?
-    dynamic_cast<c_file_ptr_buf *> (os->rdbuf ()) : 0;
+  c_file_ptr_buf *ibuf
+    = is ? dynamic_cast<c_file_ptr_buf *> (is->rdbuf ()) : 0;
+
+  c_file_ptr_buf *obuf
+    = os ? dynamic_cast<c_file_ptr_buf *> (os->rdbuf ()) : 0;
 
   int i_fid = ibuf ? ibuf->file_number () : -1;
   int o_fid = obuf ? obuf->file_number () : -1;
@@ -937,7 +930,7 @@ void
 octave_base_stream::error (const std::string& who, const std::string& msg)
 {
   fail = true;
-  errmsg = who + msg;
+  errmsg = who + ": " + msg;
 }
 
 void
@@ -977,38 +970,44 @@ octave_base_stream::do_gets (octave_idx_type max_len, bool& err,
     {
       std::istream& is = *isp;
 
-      OSSTREAM buf;
+      std::ostringstream buf;
 
       int c = 0;
       int char_count = 0;
-      int newline_stripped = 0;
 
-      while (is && (c = is.get ()) != EOF)
+      if (max_len != 0)
 	{
-	  char_count++;
-
-	  if (c == '\n')
+	  while (is && (c = is.get ()) != EOF)
 	    {
-	      if (! strip_newline)
-		buf << (char) c;
+	      char_count++;
+
+	      if (c == '\n')
+		{
+		  if (! strip_newline)
+		    buf << static_cast<char> (c);
+
+		  break;
+		}
 	      else
-		newline_stripped = 1;
+		buf << static_cast<char> (c);
 
-	      break;
+	      if (max_len > 0 && char_count == max_len)
+		break;
 	    }
-	  else
-	    buf << (char) c;
+	}
 
-	  if (max_len > 0 && char_count == max_len)
-	    break;
+      if (! is.eof () && char_count > 0)
+	{
+	  // GAGME.  Matlab seems to check for EOF even if the last
+	  // character in a file is a newline character.  This is NOT
+	  // what the corresponding C-library functions do.
+	  int disgusting_compatibility_hack = is.get ();
+	  if (! is.eof ())
+	    is.putback (disgusting_compatibility_hack);
 	}
 
       if (is.good () || (is.eof () && char_count > 0))
-	{
-	  buf << OSSTREAM_ENDS;
-	  retval = OSSTREAM_STR (buf);
-	  OSSTREAM_FREEZE (buf);
-	}
+	retval = buf.str ();
       else
 	{
 	  err = true;
@@ -1048,12 +1047,9 @@ octave_base_stream::gets (octave_idx_type max_len, bool& err, const std::string&
 
 #define OCTAVE_SCAN(is, fmt, arg) octave_scan (is, fmt, arg)
 
-// XXX FIXME XXX -- this needs to be fixed to handle formats which
-// specify a maximum width.
-
 template <class T>
 std::istream&
-octave_scan (std::istream& is, const scanf_format_elt& fmt, T* valptr)
+octave_scan_1 (std::istream& is, const scanf_format_elt& fmt, T* valptr)
 {
   T& ref = *valptr;
 
@@ -1109,6 +1105,30 @@ octave_scan (std::istream& is, const scanf_format_elt& fmt, T* valptr)
   return is;
 }
 
+template <class T>
+std::istream&
+octave_scan (std::istream& is, const scanf_format_elt& fmt, T* valptr)
+{
+  if (fmt.width)
+    {
+      // Limit input to fmt.width characters by reading into a
+      // temporary stringstream buffer.
+
+      std::string tmp;
+
+      is.width (fmt.width);
+      is >> tmp;
+
+      std::istringstream ss (tmp);
+
+      octave_scan_1 (ss, fmt, valptr);
+    }
+  else
+    octave_scan_1 (is, fmt, valptr);
+
+  return is;
+}
+
 // Note that this specialization is only used for reading characters, not 
 // character strings. See BEGIN_S_CONVERSION for details.
 
@@ -1143,9 +1163,9 @@ template std::istream&
 octave_scan (std::istream&, const scanf_format_elt&, float*);
 #endif
 
-template <>
+template<>
 std::istream&
-octave_scan (std::istream& is, const scanf_format_elt& fmt, double* valptr)
+octave_scan<> (std::istream& is, const scanf_format_elt& fmt, double* valptr)
 {
   double& ref = *valptr;
 
@@ -1255,48 +1275,49 @@ octave_scan (std::istream& is, const scanf_format_elt& fmt, double* valptr)
 			int c3 = is.get ();
 
 			if (c3 != EOF)
+			  {
+			    if (c3 == 'f')
+			      {
+				int c4 = is.get ();
 
-			  if (c3 == 'f')
-			    {
-			      int c4 = is.get ();
+				if (c4 != EOF)
+				  {
+				    is.putback (c4);
 
-			      if (c4 != EOF)
-				{
-				  is.putback (c4);
+				    if (isspace (c4) || ispunct (c4))
+				      ref = octave_Inf;
+				    else
+				      {
+					is.putback (c3);
+					is.putback (c2);
+					is.putback (c1);
+
+					is >> ref;
+				      }
+				  }
+				else
+				  {
+				    is.clear ();
 
-				  if (isspace (c4) || ispunct (c4))
 				    ref = octave_Inf;
-				  else
-				    {
-				      is.putback (c3);
-				      is.putback (c2);
-				      is.putback (c1);
+				  }
+			      }
+			    else
+			      {
+				is.putback (c3);
+				is.putback (c2);
+				is.putback (c1);
 
-				      is >> ref;
-				    }
-				}
-			      else
-				{
-				  is.clear ();
+				is >> ref;
+			      }
+			}
+		      else
+			{
+			  is.putback (c2);
+			  is.putback (c1);
 
-				  ref = octave_Inf;
-				}
-			    }
-			  else
-			    {
-			      is.putback (c3);
-			      is.putback (c2);
-			      is.putback (c1);
-
-			      is >> ref;
-			    }
-		      }
-		    else
-		      {
-			is.putback (c2);
-			is.putback (c1);
-
-			is >> ref;
+			  is >> ref;
+			}
 		      }
 		  }
 	      }
@@ -1318,9 +1339,6 @@ octave_scan (std::istream& is, const scanf_format_elt& fmt, double* valptr)
   return is;
 }
 
-template std::istream&
-octave_scan (std::istream&, const scanf_format_elt&, double*);
-
 #endif
 
 template <class T>
@@ -1457,7 +1475,7 @@ do_scanf_conv (std::istream&, const scanf_format_elt&, double*,
   int n = 0; \
  \
   while (is && n < width && (c = is.get ()) != EOF) \
-    tbuf[n++] = (char) c; \
+    tbuf[n++] = static_cast<char> (c);		    \
  \
   tbuf[n] = '\0'; \
  \
@@ -1541,7 +1559,7 @@ do_scanf_conv (std::istream&, const scanf_format_elt&, double*,
 	} \
       else \
 	{ \
-	  OSSTREAM buf; \
+	  std::ostringstream buf; \
  \
 	  std::string char_class = elt->char_class; \
  \
@@ -1551,21 +1569,19 @@ do_scanf_conv (std::istream&, const scanf_format_elt&, double*,
 	    { \
 	      while (is && (c = is.get ()) != EOF \
 		     && char_class.find (c) != NPOS) \
-		buf << (char) c; \
+		buf << static_cast<char> (c);	     \
 	    } \
 	  else \
 	    { \
 	      while (is && (c = is.get ()) != EOF \
 		     && char_class.find (c) == NPOS) \
-		buf << (char) c; \
+		buf << static_cast<char> (c);		     \
 	    } \
  \
 	  if (c != EOF) \
 	    is.putback (c); \
  \
-	  buf << OSSTREAM_ENDS; \
-	  tmp = OSSTREAM_STR (buf); \
-	  OSSTREAM_FREEZE (buf); \
+	  tmp = buf.str (); \
  \
 	  if (tmp.empty ()) \
 	    is.setstate (std::ios::failbit); \
@@ -1602,12 +1618,7 @@ do_scanf_conv (std::istream&, const scanf_format_elt&, double*,
 			    panic_impossible (); \
 			} \
 		      else if (nr > 0) \
-			{ \
-			  if (nc <= 0) \
-			    mval.resize (nr, max_size / nr, 0.0); \
-			  else \
-			    panic_impossible (); \
-			} \
+			mval.resize (nr, max_size / nr, 0.0); \
 		      else \
 			mval.resize (max_size, 1, 0.0); \
  \
@@ -1752,12 +1763,7 @@ octave_base_stream::do_scanf (scanf_format_list& fmt_list,
 			panic_impossible ();
 		    }
 		  else if (nr > 0)
-		    {
-		      if (nc <= 0)
-			mval.resize (nr, max_size / nr, 0.0);
-		      else
-			panic_impossible ();
-		    }
+		    mval.resize (nr, max_size / nr, 0.0);
 		  else
 		    mval.resize (max_size, 1, 0.0);
 
@@ -1944,7 +1950,7 @@ octave_base_stream::do_scanf (scanf_format_list& fmt_list,
 		  if (is.rdstate () & std::ios::failbit)
 		    is.clear (is.rdstate () & (~std::ios::failbit));
 
-		  // XXX FIXME XXX -- is this the right thing to do?
+		  // FIXME -- is this the right thing to do?
 
 		  if (interactive && name () == "stdin")
 		    {
@@ -2152,7 +2158,7 @@ octave_base_stream::do_oscanf (const scanf_format_elt *elt,
 	{
 	  error ("%s: read error", who.c_str ());
 
-	  // XXX FIXME XXX -- is this the right thing to do?
+	  // FIXME -- is this the right thing to do?
 
 	  if (interactive && name () == "stdin")
 	    {
@@ -2326,7 +2332,9 @@ printf_value_cache::double_value (void)
 	{
 	  octave_value tmp_val = values (val_idx);
 
-	  curr_val = tmp_val.array_value ();
+	  // Force string conversion here for compatibility.
+
+	  curr_val = tmp_val.array_value (true);
 
 	  if (! error_state)
 	    {
@@ -2480,6 +2488,45 @@ template int
 do_printf_conv (std::ostream&, const char*, int, int, int, const char*,
 		const std::string&);
 
+#define DO_DOUBLE_CONV(TQUAL) \
+  do \
+    { \
+      if (elt->modifier == 'l') \
+	{ \
+	  if (val > std::numeric_limits<TQUAL long>::max () \
+	      || val < std::numeric_limits<TQUAL long>::min ()) \
+	    { \
+	      std::string tfmt = fmt; \
+ \
+	      tfmt.replace (tfmt.rfind (elt->type), 1, ".f"); \
+	      tfmt.replace (tfmt.rfind (elt->modifier), 1, ""); \
+ \
+	      retval += do_printf_conv (os, tfmt.c_str (), nsa, sa_1, sa_2, \
+					val, who); \
+	    } \
+	  else \
+	    retval += do_printf_conv (os, fmt, nsa, sa_1, sa_2, \
+				      static_cast<TQUAL long> (val), who); \
+	} \
+      else \
+	{ \
+	  if (val > std::numeric_limits<TQUAL int>::max () \
+	      || val < std::numeric_limits<TQUAL int>::min ()) \
+	    { \
+	      std::string tfmt = fmt; \
+ \
+	      tfmt.replace (tfmt.rfind (elt->type), 1, ".f"); \
+ \
+	      retval += do_printf_conv (os, tfmt.c_str (), nsa, sa_1, sa_2, \
+					val, who); \
+	    } \
+	  else \
+	    retval += do_printf_conv (os, fmt, nsa, sa_1, sa_2, \
+				      static_cast<TQUAL int> (val), who); \
+	} \
+    } \
+  while (0)
+
 int
 octave_base_stream::do_printf (printf_format_list& fmt_list,
 			       const octave_value_list& args,
@@ -2558,68 +2605,41 @@ octave_base_stream::do_printf (printf_format_list& fmt_list,
 
 		  if (val_cache)
 		    {
-		      if (lo_ieee_is_NaN_or_NA (val) || xisinf (val)
-			  || ((val > INT_MAX || val < INT_MIN)
-			      && (elt->type == 'd'
-				  || elt->type == 'i'
-				  || elt->type == 'c'
-				  || elt->type == 'o'
-				  || elt->type == 'x'
-				  || elt->type == 'X'
-				  || elt->type == 'u')))
+		      if (lo_ieee_isnan (val) || xisinf (val))
 			{
 			  std::string tfmt = fmt;
+			  std::string::size_type i1, i2;
 
-			  if (lo_ieee_is_NaN_or_NA (val) || xisinf (val))
+			  tfmt.replace ((i1 = tfmt.rfind (elt->type)),
+					1, 1, 's');
+
+			  if ((i2 = tfmt.rfind ('.')) != NPOS && i2 < i1)
 			    {
-			      tfmt.replace (tfmt.rfind (elt->type), 1, 1, 's');
-
-			      const char *tval = xisinf (val)
-				? (val < 0 ? "-Inf" : "Inf")
-				: (lo_ieee_is_NA (val) ? "NA" : "NaN");
-
-			      retval += do_printf_conv (os, tfmt.c_str (),
-							nsa, sa_1, sa_2,
-							tval, who);
+			      tfmt.erase (i2, i1-i2);
+			      if (elt->prec < 0)
+				nsa--;
 			    }
-			  else
-			    {
-			      tfmt.replace (tfmt.rfind (elt->type), 1, ".f");
 
-			      retval += do_printf_conv (os, tfmt.c_str (),
-							nsa, sa_1, sa_2,
-							val, who);
-			    }
+			  const char *tval = xisinf (val)
+			    ? (val < 0 ? "-Inf" : "Inf")
+			    : (lo_ieee_is_NA (val) ? "NA" : "NaN");
+
+			  retval += do_printf_conv (os, tfmt.c_str (),
+						    nsa, sa_1, sa_2,
+						    tval, who);
 			}
 		      else
 			{
-			  switch (elt->type)
+			  char type = elt->type;
+
+			  switch (type)
 			    {
 			    case 'd': case 'i': case 'c':
-			      {
-				if (elt->modifier == 'l')
-				  retval += do_printf_conv
-				    (os, fmt, nsa, sa_1, sa_2,
-				     static_cast<long int> (val), who);
-				else
-				  retval += do_printf_conv
-				    (os, fmt, nsa, sa_1, sa_2,
-				     static_cast<int> (val), who);
-			      }
+			      DO_DOUBLE_CONV ( );
 			      break;
 
 			    case 'o': case 'x': case 'X': case 'u':
-			      {
-				if (elt->modifier == 'l')
-				  retval += do_printf_conv
-				    (os, fmt, nsa, sa_1, sa_2,
-				     static_cast<unsigned long int> (val),
-				     who);
-				else
-				  retval += do_printf_conv
-				    (os, fmt, nsa, sa_1, sa_2,
-				     static_cast<unsigned int> (val), who);
-			      }
+			      DO_DOUBLE_CONV (unsigned);
 			      break;
 
 			    case 'f': case 'e': case 'E':
@@ -2698,7 +2718,7 @@ octave_base_stream::puts (const std::string& s, const std::string& who)
 
       if (os)
 	{
-	  // XXX FIXME XXX -- why does this seem to be necessary?
+	  // FIXME -- why does this seem to be necessary?
 	  // Without it, output from a loop like
 	  //
 	  //   for i = 1:100, fputs (stdout, "foo\n"); endfor
@@ -2721,12 +2741,6 @@ octave_base_stream::puts (const std::string& s, const std::string& who)
   return retval;
 }
 
-int
-octave_base_stream::rewind (void)
-{
-  return seek (0, std::ios::beg);
-}
-
 // Return current error message for this stream.
 
 std::string
@@ -2747,7 +2761,7 @@ octave_base_stream::invalid_operation (const std::string& who, const char *rw)
 {
   // Note that this is not ::error () !
 
-  error (who + ": stream not open for " + rw);
+  error (who, std::string ("stream not open for ") + rw);
 }
 
 octave_stream::octave_stream (octave_base_stream *bs)
@@ -2792,7 +2806,7 @@ octave_stream::flush (void)
 {
   int retval = -1;
 
-  if (stream_ok ("fflush"))
+  if (stream_ok ())
     retval = rep->flush ();
 
   return retval;
@@ -2803,7 +2817,7 @@ octave_stream::getl (octave_idx_type max_len, bool& err, const std::string& who)
 {
   std::string retval;
 
-  if (stream_ok (who))
+  if (stream_ok ())
     retval = rep->getl (max_len, err, who);
 
   return retval;
@@ -2819,14 +2833,20 @@ octave_stream::getl (const octave_value& tc_max_len, bool& err,
 
   int conv_err = 0;
 
-  int max_len = convert_to_valid_int (tc_max_len, conv_err);
+  int max_len = -1;
 
-  if (conv_err || max_len < 0)
+  if (tc_max_len.is_defined ())
     {
-      err = true;
-      ::error ("%s: invalid maximum length specified", who.c_str ());
+      max_len = convert_to_valid_int (tc_max_len, conv_err);
+
+      if (conv_err || max_len < 0)
+	{
+	  err = true;
+	  ::error ("%s: invalid maximum length specified", who.c_str ());
+	}
     }
-  else
+
+  if (! error_state)
     retval = getl (max_len, err, who);
 
   return retval;
@@ -2837,7 +2857,7 @@ octave_stream::gets (octave_idx_type max_len, bool& err, const std::string& who)
 {
   std::string retval;
 
-  if (stream_ok (who))
+  if (stream_ok ())
     retval = rep->gets (max_len, err, who);
 
   return retval;
@@ -2853,14 +2873,20 @@ octave_stream::gets (const octave_value& tc_max_len, bool& err,
 
   int conv_err = 0;
 
-  int max_len = convert_to_valid_int (tc_max_len, conv_err);
+  int max_len = -1;
 
-  if (conv_err || max_len < 0)
+  if (tc_max_len.is_defined ())
     {
-      err = true;
-      ::error ("%s: invalid maximum length specified", who.c_str ());
+      max_len = convert_to_valid_int (tc_max_len, conv_err);
+
+      if (conv_err || max_len < 0)
+	{
+	  err = true;
+	  ::error ("%s: invalid maximum length specified", who.c_str ());
+	}
     }
-  else
+
+  if (! error_state)
     retval = gets (max_len, err, who);
 
   return retval;
@@ -2871,7 +2897,7 @@ octave_stream::seek (long offset, int origin)
 {
   int status = -1;
 
-  if (stream_ok ("fseek"))
+  if (stream_ok ())
     {
       clearerr ();
 
@@ -2978,7 +3004,7 @@ octave_stream::tell (void)
 {
   long retval = -1;
 
-  if (stream_ok ("tell"))
+  if (stream_ok ())
     retval = rep->tell ();
 
   return retval;
@@ -2987,12 +3013,7 @@ octave_stream::tell (void)
 int
 octave_stream::rewind (void)
 {
-  int retval = -1;
-
-  if (stream_ok ("frewind"))
-    retval = rep->rewind ();
-
-  return retval;
+  return seek (0, SEEK_SET);
 }
 
 bool
@@ -3000,7 +3021,7 @@ octave_stream::is_open (void) const
 {
   bool retval = false;
 
-  if (stream_ok ("is_open"))
+  if (stream_ok ())
     retval = rep->is_open ();
 
   return retval;
@@ -3009,7 +3030,7 @@ octave_stream::is_open (void) const
 void
 octave_stream::close (void)
 {
-  if (stream_ok ("close"))
+  if (stream_ok ())
     rep->close ();
 }
 
@@ -3057,7 +3078,7 @@ do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx
       max_size = 32;
     }
 
-  // XXX FIXME XXX -- byte order for Cray?
+  // FIXME -- byte order for Cray?
 
   bool swap = false;
 
@@ -3084,7 +3105,7 @@ do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx
 
       for (;;)
 	{
-	  // XXX FIXME XXX -- maybe there should be a special case for
+	  // FIXME -- maybe there should be a special case for
 	  // skip == 0.
 
 	  if (is)
@@ -3181,7 +3202,7 @@ do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx
   do_read<RET_T, READ_T> (octave_stream&, octave_idx_type, octave_idx_type, octave_idx_type, octave_idx_type, bool, \
 			  oct_mach_info::float_format, octave_idx_type&)
 
-// XXX FIXME XXX -- should we only have float if it is a different
+// FIXME -- should we only have float if it is a different
 // size from double?
 
 #define INSTANTIATE_DO_READ(VAL_T) \
@@ -3210,11 +3231,12 @@ INSTANTIATE_DO_READ (uint64NDArray);
 // INSTANTIATE_DO_READ (floatNDArray);
 INSTANTIATE_DO_READ (NDArray);
 INSTANTIATE_DO_READ (charNDArray);
+INSTANTIATE_DO_READ (boolNDArray);
 
 typedef octave_value (*read_fptr) (octave_stream&, octave_idx_type, octave_idx_type, octave_idx_type, octave_idx_type, bool,
 				   oct_mach_info::float_format ffmt, octave_idx_type&);
 
-INSTANTIATE_ARRAY (read_fptr);
+INSTANTIATE_ARRAY (read_fptr,);
 template class Array2<read_fptr>;
 
 #define FILL_TABLE_ROW(R, VAL_T) \
@@ -3267,11 +3289,11 @@ octave_stream::read (const Array<double>& size, octave_idx_type block_size,
 
   octave_value retval;
 
-  if (stream_ok ("fread"))
+  if (stream_ok ())
     {
-      // XXX FIXME XXX -- we may eventually want to make this extensible.
+      // FIXME -- we may eventually want to make this extensible.
 
-      // XXX FIXME XXX -- we need a better way to ensure that this
+      // FIXME -- we need a better way to ensure that this
       // numbering stays consistent with the order of the elements in the
       // data_type enum in the oct_data_conv class.
 
@@ -3304,7 +3326,7 @@ octave_stream::read (const Array<double>& size, octave_idx_type block_size,
 		  retval = (*fcn) (*this, nr, nc, block_size, skip,
 				   do_float_fmt_conv, ffmt, char_count);
 
-		  // XXX FIXME XXX -- kluge!
+		  // FIXME -- kluge!
 
 		  if (! error_state
 		      && (output_type == oct_data_conv::dt_char
@@ -3330,7 +3352,7 @@ octave_stream::write (const octave_value& data, octave_idx_type block_size,
 {
   octave_idx_type retval = -1;
 
-  if (stream_ok ("fwrite"))
+  if (stream_ok ())
     {
       if (! error_state)
 	{
@@ -3462,6 +3484,38 @@ do_write (std::ostream& os, const T& val, oct_data_conv::data_type output_type,
   return retval;
 }
 
+template bool
+do_write (std::ostream&, const octave_int8&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_uint8&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_int16&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_uint16&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_int32&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_uint32&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_int64&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
+template bool
+do_write (std::ostream&, const octave_uint64&, oct_data_conv::data_type,
+	  oct_mach_info::float_format, bool, bool);
+
 template <class T>
 octave_idx_type
 octave_stream::write (const Array<T>& data, octave_idx_type block_size,
@@ -3483,7 +3537,7 @@ octave_stream::write (const Array<T>& data, octave_idx_type block_size,
 
   bool do_float_conversion = (flt_fmt != native_flt_fmt);
 
-  // XXX FIXME XXX -- byte order for Cray?
+  // FIXME -- byte order for Cray?
 
   bool swap = false;
 
@@ -3507,7 +3561,7 @@ octave_stream::write (const Array<T>& data, octave_idx_type block_size,
 
 	  if (skip != 0 && (i % block_size) == 0)
 	    {
-	      // XXX FIXME XXX -- probably should try to write larger
+	      // FIXME -- probably should try to write larger
 	      // blocks...
 
 	      unsigned char zero = 0;
@@ -3605,7 +3659,7 @@ octave_stream::scanf (const std::string& fmt, const Array<double>& size,
 {
   octave_value retval;
 
-  if (stream_ok (who))
+  if (stream_ok ())
     retval = rep->scanf (fmt, size, count, who);
 
   return retval;
@@ -3641,7 +3695,7 @@ octave_stream::oscanf (const std::string& fmt, const std::string& who)
 {
   octave_value_list retval;
 
-  if (stream_ok (who))
+  if (stream_ok ())
     retval = rep->oscanf (fmt, who);
 
   return retval;
@@ -3677,7 +3731,7 @@ octave_stream::printf (const std::string& fmt, const octave_value_list& args,
 {
   int retval = -1;
 
-  if (stream_ok (who))
+  if (stream_ok ())
     retval = rep->printf (fmt, args, who);
 
   return retval;
@@ -3713,13 +3767,13 @@ octave_stream::puts (const std::string& s, const std::string& who)
 {
   int retval = -1;
 
-  if (stream_ok (who))
+  if (stream_ok ())
     retval = rep->puts (s, who);
 
   return retval;
 }
 
-// XXX FIXME XXX -- maybe this should work for string arrays too.
+// FIXME -- maybe this should work for string arrays too.
 
 int
 octave_stream::puts (const octave_value& tc_s, const std::string& who)
@@ -3746,7 +3800,7 @@ octave_stream::eof (void) const
 {
   int retval = -1;
 
-  if (stream_ok ("feof"))
+  if (stream_ok ())
     retval = rep->eof ();
 
   return retval;
@@ -3755,9 +3809,9 @@ octave_stream::eof (void) const
 std::string
 octave_stream::error (bool clear, int& err_num)
 {
-  std::string retval;
+  std::string retval = "invalid stream object";
 
-  if (stream_ok ("ferror", false))
+  if (stream_ok (false))
     retval = rep->error (clear, err_num);
 
   return retval;
@@ -3768,7 +3822,7 @@ octave_stream::name (void) const
 {
   std::string retval;
 
-  if (stream_ok ("name"))
+  if (stream_ok ())
     retval = rep->name ();
 
   return retval;
@@ -3779,7 +3833,7 @@ octave_stream::mode (void) const
 {
   int retval = 0;
 
-  if (stream_ok ("mode"))
+  if (stream_ok ())
     retval = rep->mode ();
 
   return retval;
@@ -3790,7 +3844,7 @@ octave_stream::float_format (void) const
 {
   oct_mach_info::float_format retval = oct_mach_info::flt_fmt_unknown;
 
-  if (stream_ok ("float_format"))
+  if (stream_ok ())
     retval = rep->float_format ();
 
   return retval;
@@ -3834,12 +3888,6 @@ octave_stream::mode_as_string (int mode)
   return retval;
 }
 
-void
-octave_stream::invalid_stream_error (const std::string& who) const
-{
-  ::error ("%s: attempt to use invalid I/O stream", who.c_str ());
-}
-
 octave_stream_list *octave_stream_list::instance = 0;
 
 bool
@@ -3861,7 +3909,7 @@ octave_stream_list::instance_ok (void)
 }
 
 int
-octave_stream_list::insert (const octave_stream& os)
+octave_stream_list::insert (octave_stream& os)
 {
   return (instance_ok ()) ? instance->do_insert (os) : -1;
 }
@@ -3929,40 +3977,35 @@ octave_stream_list::get_file_number (const octave_value& fid)
 }
 
 int
-octave_stream_list::do_insert (const octave_stream& os)
+octave_stream_list::do_insert (octave_stream& os)
 {
-  octave_value retval;
+  // Insert item with key corresponding to file-descriptor.
 
-  int stream_number = -1;
+  int stream_number;
 
-  // Insert item in first open slot, increasing size of list if
-  // necessary.
+  if ((stream_number = os.file_number ()) == -1)
+    return stream_number;
 
-  for (int i = 0; i < curr_len; i++)
+  // Should we test for "(list.find (stream_number) != list.end ()) &&
+  // list[stream_number].is_open ()" and respond with "error
+  // ("internal error: ...")"? It should not happen except for some
+  // bug or if the user has opened a stream with an interpreted
+  // command, but closed it directly with a system call in an
+  // oct-file; then the kernel knows the fd is free, but Octave does
+  // not know. If it happens, it should not do harm here to simply
+  // overwrite this entry, although the wrong entry might have done
+  // harm before.
+
+  if (list.size () < list.max_size ())
+    list[stream_number] = os;
+  else
     {
-      octave_stream tmp = list(i);
-
-      if (! tmp)
-	{
-	  list(i) = os;
-	  stream_number = i;
-	  break;
-	}
-    }
-
-  if (stream_number < 0)
-    {
-      int total_len = list.length ();
-
-      if (curr_len == total_len)
-	list.resize (total_len * 2);
-
-      list(curr_len) = os;
-      stream_number = curr_len;
-      curr_len++;
+      stream_number = -1;
+      error ("could not create file id");
     }
 
   return stream_number;
+
 }
 
 static void
@@ -3979,8 +4022,15 @@ octave_stream_list::do_lookup (int fid, const std::string& who) const
 {
   octave_stream retval;
 
-  if (fid >= 0 && fid < curr_len)
-    retval = list(fid);
+  if (fid >= 0)
+    {
+      ostrl_map::const_iterator iter = list.find (fid);
+
+      if (iter != list.end ())
+	retval = iter->second;
+      else
+	gripe_invalid_file_id (fid, who);
+    }
   else
     gripe_invalid_file_id (fid, who);
 
@@ -4009,15 +4059,22 @@ octave_stream_list::do_remove (int fid, const std::string& who)
   // Can't remove stdin (std::cin), stdout (std::cout), or stderr
   // (std::cerr).
 
-  if (fid > 2 && fid < curr_len)
+  if (fid > 2)
     {
-      octave_stream os = list(fid);
+      ostrl_map::iterator iter = list.find (fid);
 
-      if (os.is_valid ())
+      if (iter != list.end ())
 	{
-	  os.close ();
-	  list(fid) = octave_stream ();
-	  retval = 0;
+	  octave_stream os = iter->second;
+
+	  if (os.is_valid ())
+	    {
+	      os.close ();
+	      iter->second = octave_stream ();
+	      retval = 0;
+	    }
+	  else
+	    gripe_invalid_file_id (fid, who);
 	}
       else
 	gripe_invalid_file_id (fid, who);
@@ -4033,10 +4090,30 @@ octave_stream_list::do_remove (const octave_value& fid, const std::string& who)
 {
   int retval = -1;
 
-  int i = get_file_number (fid);
+  if (fid.is_string () && fid.string_value () == "all")
+    {
+      for (ostrl_map::iterator p = list.begin (); p != list.end (); p++)
+	{
+	  // Skip stdin, stdout, and stderr.
 
-  if (! error_state)
-    retval = do_remove (i, who);
+	  if (p->first > 2)
+	    {
+	      octave_stream os = p->second;
+
+	      if (os.is_valid ())
+		do_remove (p->first, who);
+	    }
+	}
+
+      retval = 0;
+    }
+  else
+    {
+      int i = get_file_number (fid);
+
+      if (! error_state)
+	retval = do_remove (i, who);
+    }
 
   return retval;
 }
@@ -4046,13 +4123,18 @@ octave_stream_list::do_clear (void)
 {
   // Do flush stdout and stderr.
 
-  list(0) . flush ();
-  list(1) . flush ();
+  list[0].flush ();
+  list[1].flush ();
 
   // But don't delete them or stdin.
 
-  for (int i = 3; i < curr_len; i++)
-    list(i) = octave_stream ();
+  for (ostrl_map::iterator p = list.begin (); p != list.end (); p++)
+    {
+      // Skip stdin, stdout, and stderr.
+
+      if (p->first > 2)
+	p->second = octave_stream ();
+    }
 }
 
 string_vector
@@ -4098,19 +4180,19 @@ octave_stream_list::do_list_open_files (void) const
 {
   std::string retval;
 
-  OSSTREAM buf;
+  std::ostringstream buf;
 
   buf << "\n"
       << "  number  mode  arch       name\n"
       << "  ------  ----  ----       ----\n";
 
-  for (int i = 0; i < curr_len; i++)
+  for (ostrl_map::const_iterator p = list.begin (); p != list.end (); p++)
     {
-      octave_stream os = list(i);
+      octave_stream os = p->second;
 
       buf << "  "
 	  << std::setiosflags (std::ios::right)
-	  << std::setw (4) << i << "     "
+	  << std::setw (4) << p->first << "     "
 	  << std::setiosflags (std::ios::left)
 	  << std::setw (3)
 	  << octave_stream::mode_as_string (os.mode ())
@@ -4121,11 +4203,9 @@ octave_stream_list::do_list_open_files (void) const
 	  << os.name () << "\n";
     }
 
-  buf << "\n" << OSSTREAM_ENDS;
+  buf << "\n";
 
-  retval = OSSTREAM_STR (buf);
-
-  OSSTREAM_FREEZE (buf);
+  retval = buf.str ();
 
   return retval;
 }
@@ -4133,16 +4213,16 @@ octave_stream_list::do_list_open_files (void) const
 octave_value
 octave_stream_list::do_open_file_numbers (void) const
 {
-  Matrix retval (1, curr_len, 0.0);
+  Matrix retval (1, list.size (), 0.0);
 
   int num_open = 0;
 
-  // Skip stdin, stdout, and stderr.
-
-  for (int i = 3; i < curr_len; i++)
+  for (ostrl_map::const_iterator p = list.begin (); p != list.end (); p++)
     {
-      if (list(i))
-	retval (0, num_open++) = i;
+      // Skip stdin, stdout, and stderr.
+
+      if (p->first > 2 && p->second)
+	retval(0,num_open++) = p->first;
     }
 
   retval.resize ((num_open > 0), num_open);
@@ -4159,17 +4239,20 @@ octave_stream_list::do_get_file_number (const octave_value& fid) const
     {
       std::string nm = fid.string_value ();
 
-      // stdin (std::cin), stdout (std::cout), and stderr (std::cerr)
-      // are unnamed.
-
-      for (int i = 3; i < curr_len; i++)
+      for (ostrl_map::const_iterator p = list.begin (); p != list.end (); p++)
 	{
-	  octave_stream os = list(i);
+	  // stdin (std::cin), stdout (std::cout), and stderr (std::cerr)
+	  // are unnamed.
 
-	  if (os && os.name () == nm)
+	  if (p->first > 2)
 	    {
-	      retval = i;
-	      break;
+	      octave_stream os = p->second;
+
+	      if (os && os.name () == nm)
+		{
+		  retval = p->first;
+		  break;
+		}
 	    }
 	}
     }
