@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+              2002, 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -58,8 +58,10 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "file-io.h"
 #include "input.h"
 #include "lex.h"
+#include "load-path.h"
 #include "octave.h"
 #include "oct-hist.h"
+#include "oct-map.h"
 #include "oct-obj.h"
 #include "ops.h"
 #include "toplev.h"
@@ -75,7 +77,9 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <version.h>
 
 // Kluge.
-extern "C" void F77_FUNC (xerbla, XERBLA) (const char *, octave_idx_type, long);
+extern "C" F77_RET_T
+F77_FUNC (xerbla, XERBLA) (F77_CONST_CHAR_ARG_DECL, const octave_idx_type&
+			   F77_CHAR_ARG_LEN_DECL);
 
 extern void install_builtins (void);
 
@@ -87,6 +91,13 @@ extern "C" int on_exit ();
 // The command-line options.
 static string_vector octave_argv;
 
+// The name used to invoke Octave.
+static std::string
+octave_program_invocation_name;
+
+// The last component of octave_program_invocation_name.
+static std::string octave_program_name;
+
 // TRUE means we read ~/.octaverc and ./.octaverc.
 // (--norc; --no-init-file; -f)
 static bool read_init_files = true;
@@ -95,6 +106,10 @@ static bool read_init_files = true;
 // (--norc; --no-site-file; -f)
 static bool read_site_files = true;
 
+// TRUE means we set the initial path to configured defaults.
+// (--no-initial-path)
+static bool set_initial_path = true;
+
 // TRUE means we don't print the usual startup message.
 // (--quiet; --silent; -q)
 static bool inhibit_startup_message = false;
@@ -110,10 +125,10 @@ static bool verbose_flag = false;
 // Usage message
 static const char *usage_string = 
   "octave [-?HVdfhiqvx] [--debug] [--echo-commands] [--eval CODE]\n\
-       [--exec-path path] [--help] [--info-file file] [--info-program prog]\n\
-       [--interactive] [--no-history] [--no-init-file] [--no-line-editing]\n\
-       [--no-site-file] [-p path] [--path path] [--silent] [--traditional]\n\
-       [--verbose] [--version] [file]";
+       [--exec-path path] [--help] [--image-path path] [--info-file file]\n\
+       [--info-program prog] [--interactive] [--line-editing] [--no-history] [--no-init-file]\n\
+       [--no-line-editing] [--no-site-file] [--no-init-path] [-p path]\n\
+       [--path path] [--silent] [--traditional] [--verbose] [--version] [file]";
 
 // This is here so that it's more likely that the usage message and
 // the real set of options will agree.  Note: the `+' must come first
@@ -130,13 +145,16 @@ static bool persist = false;
 // fields in this structure.
 #define EVAL_OPTION 1
 #define EXEC_PATH_OPTION 2
-#define INFO_FILE_OPTION 3
-#define INFO_PROG_OPTION 4
-#define NO_INIT_FILE_OPTION 5
-#define NO_LINE_EDITING_OPTION 6
-#define NO_SITE_FILE_OPTION 7
-#define PERSIST_OPTION 8
-#define TRADITIONAL_OPTION 9
+#define IMAGE_PATH_OPTION 3
+#define INFO_FILE_OPTION 4
+#define INFO_PROG_OPTION 5
+#define NO_INIT_FILE_OPTION 6
+#define NO_LINE_EDITING_OPTION 7
+#define NO_SITE_FILE_OPTION 8
+#define NO_INITIAL_PATH_OPTION 9
+#define PERSIST_OPTION 10
+#define TRADITIONAL_OPTION 11
+#define LINE_EDITING_OPTION 12
 long_options long_opts[] =
   {
     { "debug",            prog_args::no_arg,       0, 'd' },
@@ -145,13 +163,16 @@ long_options long_opts[] =
     { "eval",             prog_args::required_arg, 0, EVAL_OPTION },
     { "exec-path",        prog_args::required_arg, 0, EXEC_PATH_OPTION },
     { "help",             prog_args::no_arg,       0, 'h' },
+    { "image-path",       prog_args::required_arg, 0, IMAGE_PATH_OPTION },
     { "info-file",        prog_args::required_arg, 0, INFO_FILE_OPTION },
     { "info-program",     prog_args::required_arg, 0, INFO_PROG_OPTION },
     { "interactive",      prog_args::no_arg,       0, 'i' },
+    { "line-editing",     prog_args::no_arg,       0, LINE_EDITING_OPTION },
     { "no-history",       prog_args::no_arg,       0, 'H' },
     { "no-init-file",     prog_args::no_arg,       0, NO_INIT_FILE_OPTION },
     { "no-line-editing",  prog_args::no_arg,       0, NO_LINE_EDITING_OPTION },
     { "no-site-file",     prog_args::no_arg,       0, NO_SITE_FILE_OPTION },
+    { "no-initial-path",  prog_args::no_arg,       0, NO_INITIAL_PATH_OPTION },
     { "norc",             prog_args::no_arg,       0, 'f' },
     { "path",             prog_args::required_arg, 0, 'p' },
     { "persist",          prog_args::no_arg,       0, PERSIST_OPTION },
@@ -174,21 +195,15 @@ intern_argv (int argc, char **argv)
 
   nargin_sr->define (argc-1);
 
-  Cell args;
-
   if (argc > 1)
     {
-      Array<octave_value> tmp (argc-1);
+      octave_argv.resize (argc-1);
 
       // Skip program name in argv.
       int i = argc;
       while (--i > 0)
-	tmp(i-1) = octave_value (*(argv+i));
-
-      args = Cell (tmp, argc-1, 1);
+	octave_argv[i-1] = *(argv+i);
     }
-
-  bind_builtin_constant ("argv", args, true, true);
 }
 
 static void
@@ -209,6 +224,62 @@ initialize_pathsearch (void)
       + Vlibexec_dir + file_ops::dir_sep_str + "octave";
 }
 
+DEFUN (__version_info__, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {retval =} __version_info__ (@var{name}, @var{version}, @var{release}, @var{date})\n\
+Undocumented internal function.\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  static Octave_map vinfo;
+
+  int nargin = args.length ();
+
+  if (nargin == 4)
+    {
+      if (vinfo.nfields () == 0)
+	{
+	  vinfo.assign ("Name", args (0));
+	  vinfo.assign ("Version", args (1));
+	  vinfo.assign ("Release", args (2));
+	  vinfo.assign ("Date", args (3));
+	}
+      else
+	{
+	  octave_idx_type n = vinfo.numel () + 1;
+
+	  vinfo.resize (dim_vector (n, 1));
+
+	  octave_value idx (n);
+
+	  vinfo.assign (idx, "Name", Cell (octave_value (args (0))));
+	  vinfo.assign (idx, "Version", Cell (octave_value (args (1))));
+	  vinfo.assign (idx, "Release", Cell (octave_value (args (2))));
+	  vinfo.assign (idx, "Date", Cell (octave_value (args (3))));
+	}
+    }
+  else if (nargin == 0)
+    retval = vinfo;
+  else
+    print_usage ();
+
+  return retval;
+}
+
+static void
+initialize_version_info (void)
+{
+  octave_value_list args;
+
+  args(3) = OCTAVE_RELEASE_DATE;
+  args(2) = OCTAVE_RELEASE;
+  args(1) = OCTAVE_VERSION;
+  args(0) = "GNU Octave";
+
+  F__version_info__ (args, 0);
+}
+
 // Initialize by reading startup files.
 
 static void
@@ -253,7 +324,7 @@ execute_startup_files (void)
 
       std::string local_rc;
 
-      if (! home_dir.empty ())
+      if (! home_rc.empty ())
 	{
 	  parse_and_execute (home_rc, verbose);
 
@@ -270,10 +341,7 @@ execute_startup_files (void)
 
 	      local_rc = octave_env::make_absolute (initfile, curr_dir);
 
-	      file_stat fs_dot_rc (local_rc);
-
-	      if (fs_dot_rc && fs_home_rc.ino () == fs_dot_rc.ino ())
-		home_rc_already_executed = true;
+	      home_rc_already_executed = same_file (home_rc, local_rc);
 	    }
 	}
 
@@ -304,22 +372,21 @@ execute_eval_option_code (const std::string& code)
 
   int parse_status = 0;
 
-  eval_string (code, false, parse_status, 0);
+  try
+    {
+      eval_string (code, false, parse_status, 0);
+    }
+  catch (std::bad_alloc)
+    {
+      std::cerr << "error: memory exhausted or requested size too large for range of Octave's index type -- eval failed"
+		<< std::endl;
+    }
 
   unwind_protect::run_frame ("execute_eval_option_code");
 
   return parse_status;
 }
 
-static void
-restore_program_name (void *)
-{
-  bind_builtin_variable ("program_invocation_name",
-			 octave_env::get_program_invocation_name ());
-
-  bind_builtin_variable ("program_name", octave_env::get_program_name ());
-}
-
 static void
 execute_command_line_file (const std::string& fname)
 {
@@ -332,7 +399,8 @@ execute_command_line_file (const std::string& fname)
   unwind_protect_str (curr_fcn_file_name);
   unwind_protect_str (curr_fcn_file_full_name);
 
-  unwind_protect::add (restore_program_name, 0);
+  unwind_protect_str (octave_program_invocation_name);
+  unwind_protect_str (octave_program_name);
 
   interactive = false;
   reading_script_file = true;
@@ -341,16 +409,24 @@ execute_command_line_file (const std::string& fname)
   curr_fcn_file_name = fname;
   curr_fcn_file_full_name = curr_fcn_file_name;
 
-  bind_builtin_variable ("program_invocation_name", curr_fcn_file_name);
+  octave_program_invocation_name = curr_fcn_file_name;
 
   size_t pos = curr_fcn_file_name.find_last_of (file_ops::dir_sep_chars);
   
   std::string tmp = (pos != NPOS)
     ? curr_fcn_file_name.substr (pos+1) : curr_fcn_file_name;
 
-  bind_builtin_variable ("program_name", tmp);
+  octave_program_name = tmp;
 
-  parse_and_execute (fname, false, "octave");
+  try
+    {
+      parse_and_execute (fname, false, "octave");
+    }
+  catch (std::bad_alloc)
+    {
+      std::cerr << "error: memory exhausted or requested size too large for range of Octave's index type -- execution of "
+		<< fname << " failed" << std::endl;
+    }
  
   unwind_protect::run_frame ("execute_command_line_file");
 }
@@ -371,15 +447,17 @@ Options:\n\
   --eval CODE             Evaluate CODE.  Exit when done unless --persist.\n\
   --exec-path PATH        Set path for executing subprograms.\n\
   --help, -h, -?          Print short help message and exit.\n\
+  --image-path PATH       Set initial IMAGE_PATH to PATH.\n\
   --info-file FILE        Use top-level info file FILE.\n\
   --info-program PROGRAM  Use PROGRAM for reading info files.\n\
   --interactive, -i       Force interactive behavior.\n\
+  --line-editing          Force readline use for command-line editing.\n\
   --no-history, -H        Don't save commands to the history list\n\
   --no-init-file          Don't read the ~/.octaverc or .octaverc files.\n\
   --no-line-editing       Don't use readline for command-line editing.\n\
   --no-site-file          Don't read the site-wide octaverc file.\n\
   --norc, -f              Don't read any initialization files.\n\
-  --path PATH, -p PATH    Set initial LOADPATH to PATH.\n\
+  --path PATH, -p PATH    Set initial function search path to PATH.\n\
   --persist               Go interactive after --eval or reading from FILE.\n\
   --silent, -q            Don't print message at startup.\n\
   --traditional           Set compatibility variables.\n\
@@ -419,6 +497,7 @@ initialize_error_handlers ()
 {
   set_liboctave_error_handler (error);
   set_liboctave_warning_handler (warning);
+  set_liboctave_warning_with_id_handler (warning_with_id);
 }
 
 // What happens on --traditional.
@@ -428,17 +507,20 @@ maximum_braindamage (void)
 {
   persist = true;
 
-  bind_builtin_variable ("PS1", ">> ");
-  bind_builtin_variable ("PS2", "");
-  bind_builtin_variable ("beep_on_error", true);
-  bind_builtin_variable ("crash_dumps_octave_core", false);
-  bind_builtin_variable ("default_save_options", "-mat-binary");
-  bind_builtin_variable ("fixed_point_format", true);
-  bind_builtin_variable ("history_timestamp_format_string",
+  bind_internal_variable ("PS1", ">> ");
+  bind_internal_variable ("PS2", "");
+  bind_internal_variable ("beep_on_error", true);
+  bind_internal_variable ("crash_dumps_octave_core", false);
+  bind_internal_variable ("default_save_options", "-mat-binary");
+  bind_internal_variable ("fixed_point_format", true);
+  bind_internal_variable ("history_timestamp_format_string",
 			 "%%-- %D %I:%M %p --%%");
-  bind_builtin_variable ("page_screen_output", false);
-  bind_builtin_variable ("print_empty_dimensions", false);
-  bind_builtin_variable ("warn_function_name_clash", false);
+  bind_internal_variable ("page_screen_output", false);
+  bind_internal_variable ("print_empty_dimensions", false);
+
+  disable_warning ("Octave:fopen-file-in-path");
+  disable_warning ("Octave:function-name-clash");
+  disable_warning ("Octave:load-file-in-path");
 }
 
 // You guessed it.
@@ -448,12 +530,17 @@ octave_main (int argc, char **argv, int embedded)
 {
   octave_env::set_program_name (argv[0]);
 
+  octave_program_invocation_name = octave_env::get_program_invocation_name ();
+  octave_program_name = octave_env::get_program_name ();
+
   // The order of these calls is important.  The call to
   // install_defaults must come before install_builtins because
   // default variable values must be available for the variables to be
   // installed, and the call to install_builtins must come before the
   // options are processed because some command line options override
-  // defaults by calling bind_builtin_variable.
+  // defaults by calling bind_internal_variable.
+
+  init_signals ();
 
   sysdep_init ();
 
@@ -467,6 +554,8 @@ octave_main (int argc, char **argv, int embedded)
 
   initialize_error_handlers ();
 
+  initialize_default_warning_state ();
+
   install_defaults ();
 
   initialize_pathsearch ();
@@ -486,13 +575,15 @@ octave_main (int argc, char **argv, int embedded)
 
   prog_args args (argc, argv, short_opts, long_opts);
 
+  bool forced_line_editing = false;
+
   int optc;
   while ((optc = args.getopt ()) != EOF)
     {
       switch (optc)
 	{
 	case 'H':
-	  bind_builtin_variable ("saving_history", false);
+	  bind_internal_variable ("saving_history", false);
 	  break;
 
 	case 'V':
@@ -520,7 +611,7 @@ octave_main (int argc, char **argv, int embedded)
 
 	case 'p':
 	  if (args.optarg ())
-	    bind_builtin_variable ("LOADPATH", args.optarg ());
+	    load_path::set_command_line_path (args.optarg ());
 	  break;
 
 	case 'q':
@@ -530,7 +621,7 @@ octave_main (int argc, char **argv, int embedded)
 	case 'x':
 	  {
 	    double tmp = (ECHO_SCRIPTS | ECHO_FUNCTIONS | ECHO_CMD_LINE);
-	    bind_builtin_variable ("echo_executing_commands", tmp);
+	    bind_internal_variable ("echo_executing_commands", tmp);
 	  }
 	  break;
 
@@ -550,19 +641,28 @@ octave_main (int argc, char **argv, int embedded)
 
 	case EXEC_PATH_OPTION:
 	  if (args.optarg ())
-	    bind_builtin_variable ("EXEC_PATH", args.optarg ());
+	    set_exec_path (args.optarg ());
+	  break;
+
+	case IMAGE_PATH_OPTION:
+	  if (args.optarg ())
+	    set_image_path (args.optarg ());
 	  break;
 
 	case INFO_FILE_OPTION:
 	  if (args.optarg ())
-	    bind_builtin_variable ("INFO_FILE", args.optarg ());
+	    bind_internal_variable ("info_file", args.optarg ());
 	  break;
 
 	case INFO_PROG_OPTION:
 	  if (args.optarg ())
-	    bind_builtin_variable ("INFO_PROGRAM", args.optarg ());
+	    bind_internal_variable ("info_program", args.optarg ());
 	  break;
 
+        case LINE_EDITING_OPTION:
+          forced_line_editing = true;
+          break;
+
 	case NO_INIT_FILE_OPTION:
 	  read_init_files = false;
 	  break;
@@ -575,6 +675,10 @@ octave_main (int argc, char **argv, int embedded)
 	  read_site_files = 0;
 	  break;
 
+	case NO_INITIAL_PATH_OPTION:
+	  set_initial_path = false;
+	  break;
+
 	case TRADITIONAL_OPTION:
 	  traditional = true;
 	  break;
@@ -610,11 +714,13 @@ octave_main (int argc, char **argv, int embedded)
 
   octave_interpreter_ready = true;
 
-  execute_default_pkg_add_files ();
+  initialize_version_info ();
+
+  load_path::initialize (set_initial_path);
 
   execute_startup_files ();
 
-  command_history::read (false);
+  initialize_history ();
 
   if (! inhibit_startup_message && reading_startup_message_printed)
     std::cout << std::endl;
@@ -625,7 +731,7 @@ octave_main (int argc, char **argv, int embedded)
   interactive = (! embedded
 		 && isatty (fileno (stdin)) && isatty (fileno (stdout)));
 
-  if (! interactive)
+  if (! interactive && ! forced_line_editing)
     line_editing = false;
 
   // If there is an extra argument, see if it names a file to read.
@@ -645,7 +751,7 @@ octave_main (int argc, char **argv, int embedded)
       int parse_status = execute_eval_option_code (code_to_eval);
 
       if (! (persist || remaining_args > 0))
-	return (parse_status || error_state ? 1 : 0);
+	clean_up_and_exit (parse_status || error_state ? 1 : 0);
     }
 
   if (remaining_args > 0)
@@ -658,7 +764,7 @@ octave_main (int argc, char **argv, int embedded)
       execute_command_line_file (argv[last_arg_idx]);
 
       if (! persist)
-	return (error_state ? 1 : 0);
+	clean_up_and_exit (error_state ? 1 : 0);
     }
 
   // Avoid counting commands executed from startup files.
@@ -678,13 +784,19 @@ octave_main (int argc, char **argv, int embedded)
     {
       command_editor::blink_matching_paren (false);
 
-      // XXX FIXME XXX -- is this the right thing to do?
+      // FIXME -- is this the right thing to do?
 
-      bind_builtin_variable ("echo_executing_commands", ECHO_CMD_LINE);
+      bind_internal_variable ("echo_executing_commands", ECHO_CMD_LINE);
     }
 
   if (embedded)
-    return 1;
+    {
+      // FIXME -- do we need to do any cleanup here before
+      // returning?  If we don't, what will happen to Octave functions
+      // that have been registered to execute with atexit, for example?
+
+      return 1;
+    }
 
   int retval = main_loop ();
 
@@ -696,6 +808,75 @@ octave_main (int argc, char **argv, int embedded)
   return 0;
 }
 
+DEFUN (argv, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} argv ()\n\
+Return the command line arguments passed to Octave.  For example,\n\
+if you invoked Octave using the command\n\
+\n\
+@example\n\
+octave --no-line-editing --silent\n\
+@end example\n\
+\n\
+@noindent\n\
+@code{argv} would return a cell array of strings with the elements\n\
+@code{--no-line-editing} and @code{--silent}.\n\
+\n\
+If you write an executable Octave script, @code{argv} will return the\n\
+list of arguments passed to the script.  @xref{Executable Octave Programs},\n\
+for an example of how to create an executable Octave script.\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  if (args.length () == 0)
+    retval = Cell (octave_argv);
+  else
+    print_usage ();
+
+  return retval;
+}
+
+DEFUN (program_invocation_name, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} program_invocation_name ()\n\
+Return the name that was typed at the shell prompt to run Octave.\n\
+\n\
+If executing a script from the command line (e.g., @code{octave foo.m})\n\
+or using an executable Octave script, the program name is set to the\n\
+name of the script.  @xref{Executable Octave Programs}, for an example of\n\
+how to create an executable Octave script.\n\
+@seealso{program_name}\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  if (args.length () == 0)
+    retval = octave_program_invocation_name;
+  else
+    print_usage ();
+
+  return retval;
+}
+
+DEFUN (program_name, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} program_name ()\n\
+Return the last component of of the value returned by\n\
+@code{program_invocation_name}.\n\
+@seealso{program_invocation_name}\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  if (args.length () == 0)
+    retval = octave_program_name;
+  else
+    print_usage ();
+
+  return retval;
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
