@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+              2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -33,6 +33,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "lo-utils.h"
 #include "oct-alloc.h"
 #include "str-vec.h"
+#include "MatrixType.h"
 
 #include "ov-base.h"
 #include "ov-typeinfo.h"
@@ -46,18 +47,18 @@ octave_base_scalar : public octave_base_value
 public:
 
   octave_base_scalar (void)
-    : octave_base_value () { }
+    : octave_base_value (), typ (MatrixType ()) { }
 
-  octave_base_scalar (const ST& s)
-    : octave_base_value (), scalar (s) { }
+  octave_base_scalar (const ST& s, const MatrixType& t = MatrixType ())
+    : octave_base_value (), scalar (s), typ (t) { }
 
   octave_base_scalar (const octave_base_scalar& s)
-    : octave_base_value (), scalar (s.scalar) { }
+    : octave_base_value (), scalar (s.scalar), typ (s.typ) { }
 
   ~octave_base_scalar (void) { }
 
-  octave_value *clone (void) const { return new octave_base_scalar (*this); }
-  octave_value *empty_clone (void) const { return new octave_base_scalar (); }
+  octave_base_value *clone (void) const { return new octave_base_scalar (*this); }
+  octave_base_value *empty_clone (void) const { return new octave_base_scalar (); }
 
   octave_value squeeze (void) const { return scalar; }
 
@@ -81,6 +82,8 @@ public:
 
   dim_vector dims (void) const { static dim_vector dv (1, 1); return dv; }
 
+  octave_idx_type nnz (void) const { return (scalar != ST ()) ? 1 : 0; }
+
   octave_value permute (const Array<int>&, bool = false) const
     { return scalar; }
 
@@ -90,6 +93,10 @@ public:
 
   octave_value any (int = 0) const { return (scalar != ST ()); }
 
+  MatrixType matrix_type (void) const { return typ; }
+  MatrixType matrix_type (const MatrixType& _typ) const
+    { MatrixType ret = typ; typ = _typ; return ret; }
+
   bool is_scalar_type (void) const { return true; }
 
   bool is_numeric_type (void) const { return true; }
@@ -102,10 +109,16 @@ public:
 
   bool print_name_tag (std::ostream& os, const std::string& name) const;
 
+  // Unsafe.  This function exists to support the MEX interface.
+  // You should not use it anywhere else.
+  void *mex_get_data (void) const { return const_cast<ST *> (&scalar); }
+
 protected:
 
   // The value of this scalar.
   ST scalar;
+
+  mutable MatrixType typ;
 };
 
 #endif
