@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1999 John W. Eaton
+Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005, 2006, 2007
+              John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -59,6 +59,12 @@ DEFINE_OCTAVE_ALLOCATOR (octave_cell);
 
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_cell, "cell", "cell");
 
+static void
+gripe_failed_assignment (void)
+{
+  error ("assignment to cell array failed");
+}
+
 octave_value_list
 octave_cell::subsref (const std::string& type,
 		      const std::list<octave_value_list>& idx, int nargout)
@@ -137,20 +143,34 @@ octave_cell::subsasgn (const std::string& type,
 	{
 	case '(':
 	  {
-	    octave_value tmp = do_index_op (idx.front (), true);
-
-	    if (! tmp.is_defined ())
-	      tmp = octave_value::empty_conv (type.substr (1), rhs);
-
-	    if (! error_state)
+	    if (is_empty () && type[1] == '.')
 	      {
-		std::list<octave_value_list> next_idx (idx);
+		// Allow conversion of empty cell array to some other
+		// type in cases like
+		//
+		//  x = []; x(i).f = rhs
 
-		next_idx.erase (next_idx.begin ());
+		octave_value tmp = octave_value::empty_conv (type, rhs);
 
-		tmp.make_unique ();
+		return tmp.subsasgn (type, idx, rhs);
+	      }
+	    else
+	      {
+		octave_value tmp = do_index_op (idx.front (), true);
 
-		t_rhs = tmp.subsasgn (type.substr (1), next_idx, rhs);
+		if (! tmp.is_defined ())
+		  tmp = octave_value::empty_conv (type.substr (1), rhs);
+
+		if (! error_state)
+		  {
+		    std::list<octave_value_list> next_idx (idx);
+
+		    next_idx.erase (next_idx.begin ());
+
+		    tmp.make_unique ();
+
+		    t_rhs = tmp.subsasgn (type.substr (1), next_idx, rhs);
+		  }
 	      }
 	  }
 	  break;
@@ -214,8 +234,13 @@ octave_cell::subsasgn (const std::string& type,
 	      else
 		octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
 
-	    count++;
-	    retval = octave_value (this);
+	    if (! error_state)
+	      {
+		count++;
+		retval = octave_value (this);
+	      }
+	    else
+	      gripe_failed_assignment ();
 	  }
 	  break;
 
@@ -236,8 +261,13 @@ octave_cell::subsasgn (const std::string& type,
 	    else
 	      octave_base_matrix<Cell>::assign (i, Cell (t_rhs));
 
-	    count++;
-	    retval = octave_value (this);
+	    if (! error_state)
+	      {
+		count++;
+		retval = octave_value (this);
+	      }
+	    else
+	      gripe_failed_assignment ();
 	  }
 	  break;
 
@@ -433,7 +463,7 @@ octave_cell::print_raw (std::ostream& os, bool) const
 #define CELL_ELT_TAG "<cell-element>"
 
 bool 
-octave_cell::save_ascii (std::ostream& os, bool& infnan_warned)
+octave_cell::save_ascii (std::ostream& os)
 {
   dim_vector d = dims ();
   if (d.length () > 2)
@@ -451,8 +481,7 @@ octave_cell::save_ascii (std::ostream& os, bool& infnan_warned)
 	  octave_value o_val = tmp.elem (i);
 
 	  // Recurse to print sub-value.
-	  bool b = save_ascii_data (os, o_val, CELL_ELT_TAG, infnan_warned, 
-				    false, 0);
+	  bool b = save_ascii_data (os, o_val, CELL_ELT_TAG, false, 0);
 	      
 	  if (! b)
 	    return os;
@@ -474,8 +503,7 @@ octave_cell::save_ascii (std::ostream& os, bool& infnan_warned)
 	      octave_value o_val = tmp.elem (i, j);
 
 	      // Recurse to print sub-value.
-	      bool b = save_ascii_data (os, o_val, CELL_ELT_TAG, 
-					infnan_warned, false, 0);
+	      bool b = save_ascii_data (os, o_val, CELL_ELT_TAG, false, 0);
 	      
 	      if (! b)
 		return os;
@@ -1147,11 +1175,11 @@ octave_cell::as_mxArray (void) const
 
   mxArray **elts = static_cast<mxArray **> (retval->get_data ());
 
-  int nel = numel ();
+  mwSize nel = numel ();
 
   const octave_value *p = matrix.data ();
 
-  for (int i = 0; i < nel; i++)
+  for (mwIndex i = 0; i < nel; i++)
     elts[i] = new mxArray (p[i]);
 
   return retval;
