@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 2000, 2002, 2003, 2004, 2005, 2006,
+              2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -35,6 +35,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ov-list.h"
 #include "ov-struct.h"
 #include "unwind-prot.h"
+#include "utils.h"
 #include "variables.h"
 
 #include "Array-util.h"
@@ -362,7 +363,18 @@ octave_struct::subsasgn (const std::string& type,
 
 	    std::string key = key_idx(0).string_value ();
 
-	    map.assign (key, t_rhs);
+	    if (t_rhs.is_cs_list ())
+	      {
+		Cell tmp_cell = Cell (t_rhs.list_value ());
+
+		// FIXME -- shouldn't care if the dimensions of the
+		// RHS don't match the dimensions of the subscriped
+		// LHS.
+
+		map.assign (key, tmp_cell);
+	      }
+	    else
+	      map.assign (key, t_rhs);
 
 	    if (! error_state)
 	      {
@@ -548,7 +560,7 @@ the entire array.  If the cells are empty, create an empty structure\n\
 array with the specified field names.\n\
 @end deftypefn")
 {
-  octave_value_list retval;
+  octave_value retval;
 
   int nargin = args.length ();
 
@@ -560,8 +572,23 @@ array with the specified field names.\n\
   // Note that struct () creates a 1x1 struct with no fields for
   // compatibility with Matlab.
 
-  if (nargin == 1 && args(0).is_empty () && args(0).is_real_matrix ())
-    return octave_value (Octave_map (args(0).dims ()));
+  if ((nargin == 1 || nargin == 2)
+      && args(0).is_empty () && args(0).is_real_matrix ())
+    {
+      Cell fields;
+
+      if (nargin == 2)
+	{
+	  if (args(1).is_cellstr ())
+	    retval = Octave_map (args(0).dims (), args(1).cell_value ());
+	  else
+	    error ("struct: expecting cell array of field names as second argument");
+	}
+      else
+	retval = Octave_map (args(0).dims ());
+
+      return retval;
+    }
     
   // Check for "field", VALUE pairs.
 
@@ -616,6 +643,12 @@ array with the specified field names.\n\
       if (error_state)
 	return retval;
 
+      if (! valid_identifier (key))
+	{
+	  error ("struct: invalid structure field name `%s'", key.c_str ());
+	  return retval;
+	}
+
       // Value may be v, { v }, or { v1, v2, ... }
       // In the first two cases, we need to create a cell array of
       // the appropriate dimensions filled with v.  In the last case, 
@@ -640,7 +673,7 @@ array with the specified field names.\n\
       if (error_state)
 	return retval;
     }
-  
+
   return octave_value (map);
 }
 
@@ -941,6 +974,13 @@ A(1)\n\
 				return retval;
 			    }
 
+			  if (! valid_identifier (field_str))
+			    {
+			      error ("cell2struct: invalid field name `%s'",
+				     field_str.c_str ());
+			      break;
+			    }
+
 			  map.reshape (value_dv);
 
 			  map.assign (field_str, c_value);
@@ -1017,7 +1057,7 @@ named fields.\n\
 }
 
 bool
-octave_struct::save_ascii (std::ostream& os, bool& infnan_warned)
+octave_struct::save_ascii (std::ostream& os)
 {
   Octave_map m = map_value ();
   os << "# length: " << m.nfields () << "\n";
@@ -1027,7 +1067,7 @@ octave_struct::save_ascii (std::ostream& os, bool& infnan_warned)
     {
       octave_value val = map.contents (i);
 
-      bool b = save_ascii_data (os, val, m.key (i), infnan_warned, false, 0);
+      bool b = save_ascii_data (os, val, m.key (i), false, 0);
       
       if (! b)
 	return os;
@@ -1271,9 +1311,9 @@ octave_struct::as_mxArray (void) const
 
   mxArray **elts = static_cast<mxArray **> (retval->get_data ());
 
-  int nel = numel ();
+  mwSize nel = numel ();
 
-  int ntot = nf * nel;
+  mwSize ntot = nf * nel;
 
   for (int i = 0; i < nf; i++)
     {
@@ -1281,8 +1321,8 @@ octave_struct::as_mxArray (void) const
 
       const octave_value *p = c.data ();
 
-      int k = 0;
-      for (int j = i; j < ntot; j += nf)
+      mwIndex k = 0;
+      for (mwIndex j = i; j < ntot; j += nf)
 	elts[j] = new mxArray (p[k++]);
     }
 
